-- hds header_start
--
-- VHDL Entity TIM_fft.fourier.symbol
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 10:29:15 04/12/05
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

-- -----------------------------------------------------------------------------
--    ENTITY
-- -----------------------------------------------------------------------------
-- 
entity fourier is
   port( 
      clk        : in     std_logic;
      reset      : in     std_logic;
      FourierSel : in     std_logic;
      freqbusy_o : out    std_logic;
      vmeRD      : in     std_logic;
      vmeWR      : in     std_logic;
      trig_OA_i  : in     std_logic;
      ab         : in     std_logic_vector (7 downto 0);
      db         : inout  std_logic_vector (15 downto 0);
      
      --  Spare Signals
      DebugBus   : out    std_logic_vector (15 downto 0)
   );

-- Declarations

end fourier ;


-- hds interface_end

architecture rtl of fourier is

-------------------------------------------------------------------------------
--	SIGNAL DECLARATIONS
-------------------------------------------------------------------------------
--       window 0              window 1
-- |--------------------|--------------------|-----------
-- |msb trig_buff(0) lsb|msb trig_buff(1) lsb|

	
	-- Counting from zero:
	constant WINDOW_WIDTH	: integer := 31;
	constant WINDOWS_MAX    : integer := 15; -- lowest freq
	constant WINDOWS_MIN    : integer := 8;  -- highest freq 
	--constant TRIG_BUF_LEN   : integer := ((WINDOW_WIDTH+1) * (WINDOWS_MAX+1))-1;


	subtype   word_win   	is std_logic_vector((WINDOW_WIDTH) downto 0);
	type      word_wins  	is array (WINDOWS_MAX downto WINDOWS_MIN) of word_win; 
	type      word_wins_tb 	is array (WINDOWS_MAX downto 0) of word_win; 
	
	subtype   word16 	    is std_logic_vector(15 downto 0);
	type      word_win_16 is array (WINDOWS_MAX downto WINDOWS_MIN) of word16;

	signal	trig				: std_logic;


	signal   trig_buff		: word_wins_tb;
	signal   trig_sel   		: word_wins;
	signal   selector			: word_wins;

	signal   trig_bin	   	: word_win_16;
		

	signal   Register3A   	: std_logic_vector(15 downto 0);
	signal   Register3C   	: std_logic_vector(15 downto 0);
	signal   Register3E   	: std_logic_vector(15 downto 0);

-------------------------------------------------------------------------------
--	MAIN CODE
-------------------------------------------------------------------------------
begin

-- Bits    Clocks		 Period(ns)	    Freq(Hz)
--  1			    2					50		20,000,000
--  2			    4				  100		10,000,000
--  3			    8				  200		 5,000,000
--  4			   16				  400		 2,500,000
--  5			   32				  800		 1,250,000
--  6			   64				 1600		  	625,000
--  7			  128				 3200		  	312,500
--  8			  256				 6400		  	156,250
--  9			  512				12800		  	 78,125
-- 10			 1024				25600		  	 39,062.5
-- 11			 2048				51200		  	 19,531.3
-- 12			 4096			  102400	     	  9,765.6
-- 13			 8192			  204800	     	  4,882.8
-- 14			16384			  409600	     	  2,441.4
-- 15			32768			  819200	     	  1,220.7
-- 16			65536			 1638400	          610.4
-- 17		  131072			 3276800				 305.1
-- 18		  262144			 6553600				 152.5
-- 19		  524288			13107200				  76.2
-- 20		 1048576			26214400				  38.1
-- 21		 2097152			52428800				  19.0
-- 22		 4194304		  104857600			      9.5
-- 23		 8388608		  209715200			      4.7
-- 24		16777216		  419430400				   2.3
-- 25		33554432		  838860800				   1.1
-- 26		67108864		 1677721600				   0.6


	trig <= trig_OA_i;
	--freqbusy_o <= tfbusy;

	------------------------------------------------------------
	trig_buff_sr_max : process (clk)
	begin
		if	rising_edge(clk) then
			--if	(reset = '1') then
			--		trig_buff(0) <= (others => '0');

			--else
				trig_buff(0)(WINDOW_WIDTH) <= trig;
				
				trig_buff(0)((WINDOW_WIDTH-1) downto 0) 
					<= trig_buff(0)(WINDOW_WIDTH downto 1);

			--end if;
		end if;
	end process;
	
	trig_buff_sr_gen : for n in 1 to WINDOWS_MAX generate
	begin
		process (clk)
		begin
			if	rising_edge(clk) then
				--if	(reset = '1') then
				--	trig_buff(n) <= (others => '0');
				--else
					trig_buff(n)((WINDOW_WIDTH-1) downto 0) 
						<= trig_buff(n)(WINDOW_WIDTH downto 1);
					
					trig_buff(n)(WINDOW_WIDTH) <= trig_buff(n-1)(0);
					
				--end if;
			end if;
		end process;
	end generate;
	-----------------------------------------------------------

	selector_array : for n in (WINDOWS_MIN) to (WINDOWS_MAX) generate
	begin
		selector(n)	<= (others => '1');
	end generate;

	------------------------------------------------------------

	trig_sel_array : for n in WINDOWS_MIN to WINDOWS_MAX generate
	begin
		trig_sel(n) <= trig_buff(n) and selector(n);
	end generate;

	
	trigger_bins_generate: for n in WINDOWS_MIN to WINDOWS_MAX generate
	begin
		process (clk)
		begin
	
			if	rising_edge(clk) then
				--if	(reset = '1') then
				--	trig_bin(n) <= (others => '0');
	
				--els
				if (trig = '1') then
					trig_bin(n) <= trig_bin(n)
										+ trig_sel(n)(31) --
										+ trig_sel(n)(30) --
										+ trig_sel(n)(29) --
										+ trig_sel(n)(28) --
										+ trig_sel(n)(27) --
										+ trig_sel(n)(26) --
										+ trig_sel(n)(25) --
										+ trig_sel(n)(24) --
										+ trig_sel(n)(23) --
										+ trig_sel(n)(22) --
										+ trig_sel(n)(21) --
										+ trig_sel(n)(20) --
										+ trig_sel(n)(19) --
										+ trig_sel(n)(18) --
										+ trig_sel(n)(17) --
										+ trig_sel(n)(16) --
										+ trig_sel(n)(15) --
										+ trig_sel(n)(14) --
										+ trig_sel(n)(13) --
										+ trig_sel(n)(12) --
										+ trig_sel(n)(11) --
										+ trig_sel(n)(10) --
										+ trig_sel(n)( 9) --
										+ trig_sel(n)( 8) --
										+ trig_sel(n)( 7) --
										+ trig_sel(n)( 6) --
										+ trig_sel(n)( 5) --
										+ trig_sel(n)( 4) --
										+ trig_sel(n)( 3) --
										+ trig_sel(n)( 2) --
										+ trig_sel(n)( 1) --
										+ trig_sel(n)( 0);--
				end if;
			end if;
		end process;
	end generate;




	--	Registers
	----------------------------------------------------

 	-- 3A -
 	--
 	--

	--	3C -
	--
 	--

   -- 3E -
   --
	--


--	process (reset, clk)
--	begin
--		if rising_edge(clk) then
--
--			if	(reset = '1') then                       -- Defaults:
--				Register3A <= "1111111000001001"; -- lsb slop, 9 count busy
--				Register3C <= "0000000011111111"; -- All
--				Register3E <= "0000000000000100"; -- 4 clk busy duration
--
--			else
--				if	   (Reg3A and vmeWR)	= '1'	then
--					Register3A <= db;
--
--				elsif	(Reg3C and vmeWR)	= '1'	then
--					Register3C <= db;
--
--				elsif	(Reg3E and vmeWR)	= '1'	then
--					Register3E <= db;
--
--				end if;
--			end if;
--		end if;
--	end process;

	process (reset, clk)
		variable ab_int : integer range 0 to 255; 
	begin
  	
  		ab_int := CONV_INTEGER(ab);
  		if rising_edge(clk) then

  			if (reset = '1') then
  				db <= (others => 'Z');
  		
	    	elsif (FourierSel = '1') and (vmeRD = '1') then
				db <= trig_bin(ab_int);
	      	
      	else
				db <= (others => 'Z');
				
			end if;
		end if;
  	end process;

	--	Data Bus
	----------------------------------------------------

--	db <= Register3A 	when ((Reg3A and vmeRD) = '1') else
--		   Register3C  when ((Reg3C and vmeRD) = '1') else
--		   Register3E 	when ((Reg3E and vmeRD) = '1') else
--		   (others => 'Z');



   -- Debug Bus
  	----------------------------------------------------

	DebugBus <= (others => '0');

end rtl;

-------------------------------------------------------------------------------