-- hds header_start
--
-- VHDL Architecture TIM_fft.tfb2_control.rtl
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 12:05:31 02/15/04
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end
LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity tfb2_control is
   port( 
      rst                   : in     std_logic;
      clk                   : in     std_logic;
      db                    : inout  std_logic_vector (15 downto 0);
      reg40                 : in     std_logic;
      reg42                 : in     std_logic;
      reg44                 : in     std_logic;
      reg46                 : in     std_logic;
      reg48                 : in     std_logic;
      reg4A                 : in     std_logic;
      vmeRD                 : in     std_logic;
      vmeWR                 : in     std_logic;
      period_min_o          : out    std_logic_vector (15 downto 0);
      period_max_o          : out    std_logic_vector (15 downto 0);
      period_oflow_o        : out    std_logic_vector (15 downto 0);
      period_deltat_max_o   : out    std_logic_vector (15 downto 0);
      period_match_thresh_o : out    std_logic_vector (15 downto 0);
      busy_duration_o       : out    std_logic_vector (15 downto 0)
   );

-- Declarations

end tfb2_control ;

-- hds interface_end
architecture rtl of tfb2_control is

   signal Reg_TFB_P_Min 			: std_logic_vector(15 downto 0);
   signal Reg_TFB_P_Max 			: std_logic_vector(15 downto 0);
   signal Reg_TFB_P_OFlow 			: std_logic_vector(15 downto 0);
	signal Reg_TFB_P_DeltaT_Max 	: std_logic_vector(15 downto 0);
	signal Reg_TFB_P_Match_Thresh : std_logic_vector(15 downto 0);
	signal Reg_TFB_Busy_Duration	: std_logic_vector(15 downto 0);
	
	signal reg_decode					: std_logic_vector(6 downto 0);
	
	signal dummy_bit	: std_logic;

begin

	period_min_o          <= Reg_TFB_P_Min;
	period_max_o          <= Reg_TFB_P_Max;
	period_oflow_o        <= Reg_TFB_P_OFlow;
	period_deltat_max_o   <= Reg_TFB_P_DeltaT_Max;
	period_match_thresh_o <= Reg_TFB_P_Match_Thresh; 
	busy_duration_o       <= Reg_TFB_Busy_Duration;


   --   Registers
   ----------------------------------------------------

    -- 40 - Period Min
    -- 42 - Period Max
	 -- 44 - Period DeltaT Max
	 -- 46 - Period Match Threshhold
	 -- 48 - Busy Duration 

	reg_decode <= (vmeWR & reg40 & reg42 & reg44 & reg46 & reg48 & reg4A);

   process (rst, clk)
   begin
      if rising_edge(clk) then
      
         if   (rst = '1') then
	         Reg_TFB_P_Min          <= "0000100010100000"; --2208 = ~18kHz
				Reg_TFB_P_Max          <= "0000101100100000"; --2848 = ~14kHz 
				Reg_TFB_P_OFlow        <= "0000111111111111"; --4095 = ~40kHz
				Reg_TFB_P_DeltaT_Max   <= "0000000001111111"; -- 127 = +/- ~500Hz
				Reg_TFB_P_Match_Thresh <= "0000000000000101"; -- 5 
				Reg_TFB_Busy_Duration  <= "0010000000000000"; -- 8000?
   
         else
         	if (vmeWR = '1') then
	         	case reg_decode is
            		when "1100000" => Reg_TFB_P_Min          <= db;
            		when "1010000" => Reg_TFB_P_Max          <= db;
            		when "1001000" => Reg_TFB_P_OFlow        <= db;
            		when "1000100" => Reg_TFB_P_DeltaT_Max   <= db;
            		when "1000010" => Reg_TFB_P_Match_Thresh <= db;
            		when "1000001" => Reg_TFB_Busy_Duration  <= db;
            		
            		when others   => dummy_bit <= '0';
   
            	end case;
            end if;
         end if;
      end if;
   end process;


   --   Data Bus
   ----------------------------------------------------
   db <= Reg_TFB_P_Min				when ((reg40 and vmeRD) = '1') else
			Reg_TFB_P_Max				when ((reg42 and vmeRD) = '1') else
			Reg_TFB_P_OFlow			when ((reg44 and vmeRD) = '1') else
			Reg_TFB_P_DeltaT_Max 	when ((reg46 and vmeRD) = '1') else
			Reg_TFB_P_Match_Thresh  when ((reg48 and vmeRD) = '1') else
			Reg_TFB_Busy_Duration   when ((reg4A and vmeRD) = '1') else
			
        (others => 'Z');


end rtl;

------------------------------------------------------------
-- Bits    Clocks       Period(ns)       Freq(Hz)
--  1             2               50      20,000,000
--  2             4              100      10,000,000
--  3             8              200       5,000,000
--  4            16              400       2,500,000
--  5            32              800       1,250,000
--  6            64             1600           625,000
--  7           128             3200           312,500
--  8           256             6400           156,250
--  9           512            12800            78,125
-- 10          1024            25600            39,062.5
-- 11          2048            51200            19,531.3
-- 12          4096           102400             9,765.6
-- 13          8192           204800             4,882.8
-- 14         16384           409600             2,441.4
-- 15         32768           819200             1,220.7
-- 16         65536          1638400             610.4
-- 17        131072          3276800             305.1
-- 18        262144          6553600             152.5
-- 19        524288         13107200              76.2
-- 20       1048576         26214400              38.1
-- 21       2097152         52428800              19.0
-- 22       4194304        104857600               9.5
-- 23       8388608        209715200               4.7
-- 24      16777216        419430400               2.3
-- 25      33554432        838860800               1.1
-- 26      67108864       1677721600               0.6



