-- hds header_start
--
-- VHDL Architecture TIM_fft.basictfb.rtl
--
-- Created (again):
--          by - warren.UNKNOWN (ELECTRON)
--          at - 11:46:48 02/12/04
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity tfb2 is
   port( 
      rst                   : in     std_logic;
      clk                   : in     std_logic;
      trigger_i             : in     std_logic;
      busy_o                : out    std_logic;
      period_min_i          : in     std_logic_vector (15 downto 0);
      period_max_i          : in     std_logic_vector (15 downto 0);
      period_oflow_i        : in     std_logic_vector (15 downto 0);
      period_deltat_max_i   : in     std_logic_vector (15 downto 0);
      busy_duration_i       : in     std_logic_vector (15 downto 0);
      period_match_thresh_i : in     std_logic_vector (15 downto 0)
   );

-- Declarations

end tfb2 ;

-- hds interface_end

architecture rtl of tfb2 is

-- Bits    Clocks		 Period(ns)	    Freq(Hz)
--  1			    2					50		20,000,000
--  2			    4				  100		10,000,000
--  3			    8				  200		 5,000,000
--  4			   16				  400		 2,500,000
--  5			   32				  800		 1,250,000
--  6			   64				 1600		  	625,000
--  7			  128				 3200		  	312,500
--  8			  256				 6400		  	156,250
--  9			  512				12800		  	 78,125
-- 10			 1024				25600		  	 39,062.5
-- 11			 2048				51200		  	 19,531.3
-- 12			 4096			  102400	     	  9,765.6
-- 13			 8192			  204800	     	  4,882.8
-- 14			16384			  409600	     	  2,441.4
-- 15			32768			  819200	     	  1,220.7
-- 16			65536			 1638400	          610.4
-- 17		  131072			 3276800				 305.1
-- 18		  262144			 6553600				 152.5
-- 19		  524288			13107200				  76.2
-- 20		 1048576			26214400				  38.1
-- 21		 2097152			52428800				  19.0
-- 22		 4194304		  104857600			      9.5
-- 23		 8388608		  209715200			      4.7
-- 24		16777216		  419430400				   2.3
-- 25		33554432		  838860800				   1.1
-- 26		67108864		 1677721600				   0.6


	signal p_buffer  				: std_logic_vector(15 downto 0);
	signal p_count  				: std_logic_vector(15 downto 0);

	
	signal p_count_min			: std_logic_vector(15 downto 0);
	signal p_count_max			: std_logic_vector(15 downto 0);
	signal p_count_oflow			: std_logic_vector(15 downto 0);
	signal p_under					: std_logic;
	signal p_top					: std_logic;
	signal p_over					: std_logic;
	signal p_oflow				   : std_logic;
	
	signal p_diff					: std_logic_vector(15 downto 0);
	signal p_deltat				: std_logic_vector(15 downto 0);
	signal p_deltat_max   	   : std_logic_vector(15 downto 0);
	
	signal p_match					: std_logic;
	signal p_match_count 		: std_logic_vector(15 downto 0);
	signal p_match_count_max	: std_logic_vector(15 downto 0);
	signal p_match_count_zero	: std_logic;
	
	signal trig						: std_logic;
	signal goodtrig				: std_logic;
	signal goodtrig_q				: std_logic;
	
	signal deltat_ok				: std_logic;
	
	signal busy_set				: std_logic;
	
	signal busy						: std_logic;
	signal busy_count				: std_logic_vector(15 downto 0);
	signal busy_count_max		: std_logic_vector(15 downto 0);
	

-------------------------------------------------------------------------------
--	MAIN CODE
-------------------------------------------------------------------------------

begin

   p_count_max       <= period_max_i;
   p_count_min       <= period_min_i;
   p_count_oflow  	<= period_oflow_i;
   p_deltat_max   	<= period_deltat_max_i;
   p_match_count_max <= period_match_thresh_i;
   busy_count_max  	<= busy_duration_i;

	trig 					<= trigger_i;
	busy_o 				<= busy;

	------------------------------------------------------------
   p_under <= '1' when ((p_count < p_count_min) and (p_over = '0')) else '0';
   p_top   <= '1' when ((p_count = p_count_max) and (p_over = '0')) else '0';

   goodtrig <= trig and (not p_under);
   
	
	-------------------------------------------------------------
	period_counter : process (clk)
	begin

		if	rising_edge(clk) then
			if	(rst = '1') then
				p_count <= (others => '0');
				p_over  <= '0';
				p_oflow <= '0';
				
			else
				--defaults
				p_oflow <= '0';
			
				if (goodtrig = '1') then
					p_count <= (others => '0');
					p_over <= '0';
					
				elsif (p_top = '1') then
					p_over <= '1';
					
				elsif (p_count = p_count_oflow) then
					p_oflow <= '1';
					p_count <= (others => '0');
					
				else
					p_count <= p_count + '1';					
	
				end if;
			end if;	
		end if;
	end process;


	------------------------------------------------------------
	period_buffer : process (clk)
	begin

		if	rising_edge(clk) then
			if	(rst = '1') then
				p_buffer <= (others => '0');

			elsif (goodtrig = '1') then
				if (p_over = '1') then
					p_buffer <= (others => '0');
					
				else
					p_buffer <= p_count;
				
				end if;
			end if;
		end if;
	end process;


	-----------------------------------------------------------
	
	-- lots of combinatorial, let's hope it fits nicely!
	
	p_diff 	<= (p_buffer - p_count);
	
	p_deltat <= p_diff 	when (p_diff(15) = '0') 			else (not p_diff);
	p_match	<= '1' 		when (p_deltat < p_deltat_max) 	else '0';
	
	p_match_count_zero <= '1' when (p_match_count = "0000000000000000") else '0';

	deltat_ok <= p_match and goodtrig;
					
	
	period_match_counter : process (clk)
	begin
		if	rising_edge(clk) then

			if	(rst = '1') then
				p_match_count <= (others => '0');	

			elsif (goodtrig = '1') then
			
				if (p_match = '1') then
  					p_match_count <= p_match_count + '1';
				
				elsif (p_match_count_zero = '0') then
					p_match_count <= p_match_count - '1';
				
				end if;
				
			-- decriments match_count everytime period counter overflows!	
			elsif (p_oflow = '1') and (p_match_count_zero = '0') then
				p_match_count <= p_match_count - '1';
						
			end if;
		end if;
	end process;
	

	------------------------------------------------------------------

	busy_set_gen : process (clk)
	begin
		if	rising_edge(clk) then
			if	(rst = '1') then
				busy_set   <= '0';
				goodtrig_q <= '0';
				
			else
				goodtrig_q <= goodtrig;
				
				if (goodtrig_q = '1') then
					if (p_match_count >= p_match_count_max) then
						busy_set <= '1';
					end if;
					
				else
					busy_set <= '0';
				
				end if;
			end if;
		end if;
	end process;


	
	busy_signal : process (clk)
	begin

		if	rising_edge(clk) then
			if	(rst = '1') then
				busy <= '0';
				busy_count 	<= (others => '0');

			elsif (busy_set = '1') then
					busy <= '1';
					busy_count <= (others => '0');

			elsif (busy_count = busy_count_max ) then
					busy <= '0';
					busy_count <= (others => '0');

			elsif (busy = '1') then
					busy_count <= busy_count + '1';

			else
				busy <= '0';
				busy_count <= (others => '0');

			end if;
		end if;
	end process;


end rtl;

-------------------------------------------------------------------------------