-- hds header_start
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity fftv_reg is
   port( 
      rst                 : in     std_logic;
      clk                 : in     std_logic;
      db_rd               : in     std_logic;
      db_wr               : in     std_logic;
      ren_fftv_count_lo_i : in     std_logic;
      ren_fftv_count_hi_i : in     std_logic;
      ren_fftv_count_ex_i : in     std_logic;
      db_io               : inout  std_logic_vector (15 downto 0);
      reg_fftv_p_min_i    : in     std_logic_vector (15 downto 0);
      reg_fftv_p_max_i    : in     std_logic_vector (15 downto 0);
      reg_fftv_p_oflow_i  : in     std_logic_vector (15 downto 0);
      reg_fftv_p_deltat_i : in     std_logic_vector (15 downto 0);
      reg_fftv_vetolen_i  : in     std_logic_vector (15 downto 0);
      reg_fftv_matchlvl_i : in     std_logic_vector (15 downto 0);
      ren_fv_id_hi_i      : in     std_logic;
      ren_fv_id_lo_i      : in     std_logic;
      trigger_i           : in     std_logic;
      veto_o              : out    std_logic;
      fftv_disable_link   : in     std_logic;
      runmode_i           : in     std_logic;
      fv_disable_i        : in     std_logic
   );

-- Declarations

end fftv_reg ;


-- hds interface_end

architecture rtl_tim of fftv_reg is

--Bits      Clocks	  Period(ns)     Freq(Hz)
--  1			    2					50		20,000,000
--  2			    4				  100		10,000,000
--  3			    8				  200		 5,000,000
--  4			   16				  400		 2,500,000
--  5			   32				  800		 1,250,000
--  6			   64				 1600		  	625,000
--  7			  128				 3200		  	312,500
--  8			  256				 6400		  	156,250
--  9			  512				12800		  	 78,125
-- 10			 1024				25600		  	 39,062.5
-- 11			 2048				51200		  	 19,531.3
-- 12			 4096			  102400	     	  9,765.6
-- 13			 8192			  204800	     	  4,882.8
-- 14			16384			  409600	     	  2,441.4
-- 15			32768			  819200	     	  1,220.7
-- 16			65536			 1638400	          610.4
-- 17		  131072			 3276800				 305.1
-- 18		  262144			 6553600				 152.5
-- 19		  524288			13107200				  76.2
-- 20		 1048576			26214400				  38.1
-- 21		 2097152			52428800				  19.0
-- 22		 4194304		  104857600			      9.5
-- 23		 8388608		  209715200			      4.7
-- 24		16777216		  419430400				   2.3
-- 25		33554432		  838860800				   1.1
-- 26		67108864		 1677721600				   0.6
-- 27   134217728     3355443200             0.3
-- 28   268435456     6710886400             0.15
-- 29   536870912    13421772800             0.08
-- 30  1073741824    26843545600             0.04
-- 31  2147483648    53687091200             0.02




	signal p_buffer  				: std_logic_vector(15 downto 0);
	signal p_count  				: std_logic_vector(15 downto 0);

	
	signal p_count_min			: std_logic_vector(15 downto 0);
	signal p_count_max			: std_logic_vector(15 downto 0);
	signal p_count_oflow			: std_logic_vector(15 downto 0);
	signal p_under					: std_logic;
	signal p_top					: std_logic;
	signal p_over					: std_logic;
	signal p_oflow				   : std_logic;
	
	signal p_diff					: std_logic_vector(15 downto 0);
	signal p_deltat				: std_logic_vector(15 downto 0);
	signal p_deltat_max   	   : std_logic_vector(15 downto 0);
	
	signal p_match					: std_logic;
	signal p_match_count 		: std_logic_vector(15 downto 0);
	signal p_match_count_max	: std_logic_vector(15 downto 0);
	signal p_match_count_zero	: std_logic;
	
	signal trig						: std_logic;
	signal goodtrig				: std_logic;
	signal goodtrig_q				: std_logic;
	
	signal deltat_ok				: std_logic;
	
	signal veto_set				: std_logic;
	
	signal veto						: std_logic;
	signal veto_en					: std_logic;
	signal vetolen_count			: std_logic_vector(15 downto 0);
	signal vetolen_count_max	: std_logic_vector(15 downto 0);

	signal veto_count				: std_logic_vector(47 downto 0);
	signal wr_reg_veto_count	: std_logic;
	
	
	signal fv_id					: std_logic_vector(31 downto 0);
	signal rgw_fv_id				: std_logic;
	

-------------------------------------------------------------------------------
--	MAIN CODE
-------------------------------------------------------------------------------

begin

   p_count_max       <= reg_fftv_p_max_i;
   p_count_min       <= reg_fftv_p_min_i;
   p_count_oflow  	<= reg_fftv_p_oflow_i;
   p_deltat_max   	<= reg_fftv_p_deltat_i;
   p_match_count_max <= reg_fftv_matchlvl_i;
   vetolen_count_max <= reg_fftv_vetolen_i;
   
  	veto_en 				<= ( not(fftv_disable_link) or not(fv_disable_i) ) 
  								and not(runmode_i);
	
	veto_o 				<= veto;
	
	trig 					<= trigger_i and veto_en;
	
	
	-- Period Counter and flags
	------------------------------------------------------------
   p_under <= '1' when ((p_count < p_count_min) and (p_over = '0')) else '0';
   p_top   <= '1' when ((p_count = p_count_max) and (p_over = '0')) else '0';

   goodtrig <= trig and not(p_under);
   	
	period_counter : process (clk)
	begin

		if	rising_edge(clk) then
			if	(rst = '1') or (veto_en = '0') then
				p_count <= (others => '0');
				p_over  <= '0';
				p_oflow <= '0';
				
			else
				--defaults
				p_oflow <= '0';
			
				if (goodtrig = '1') then
					p_over <= '0';
					p_count <= (others => '0');
					
				elsif (p_top = '1') then
					p_over <= '1';
					p_count <= p_count + '1';					
					
				elsif (p_count = p_count_oflow) then
					p_oflow <= '1';
					p_count <= (others => '0');
					
				else
					p_count <= p_count + '1';					
	
				end if;
			end if;	
		end if;
	end process;


	-- Previous Period Buffer
	------------------------------------------------------------
	period_buffer : process (clk)
	begin

		if	rising_edge(clk) then              -- *** added to ensure we get a proper match after a veto
			if	(rst = '1') or (veto_en = '0') or (veto = '1') then
				p_buffer <= (others => '0');

			elsif (goodtrig = '1') then
				p_buffer <= p_count;
				
			end if;
		end if;
	end process;


	-- Period Match Count
	-----------------------------------------------------------
	-- lots of combinatorial, let's hope it fits nicely!

	
	
	p_deltat	<= (p_buffer - p_count) when (p_buffer > p_count) 
			 else (p_count - p_buffer);

	--p_deltat <= p_diff 	when (p_diff(15) = '0') 			else (not p_diff);
	p_match	<= '1' 		when (p_deltat <= p_deltat_max) 	else '0';
	
	p_match_count_zero <= '1' when (p_match_count = "0000000000000000") else '0';

	deltat_ok <= p_match and goodtrig;
		
	period_match_counter : process (clk)
	begin
		if	rising_edge(clk) then

			if	(rst = '1') or (veto_en = '0') then
				p_match_count <= (others => '0');

			elsif (goodtrig = '1') then
			
				if (p_match = '1') and (p_oflow = '0') then
  					p_match_count <= p_match_count + '1';
				
				elsif (p_match_count_zero = '0') then
					p_match_count <= p_match_count - '1';
				
				end if;
				
			-- decriments match_count everytime period counter overflows!
			elsif (p_oflow = '1') and (p_match_count_zero = '0') then
				p_match_count <= p_match_count - '1';
						
			end if;
		end if;
	end process;
	


	-- Generater Veto
	------------------------------------------------------------------
	veto_set_gen : process (clk)
	begin
		if	rising_edge(clk) then
			if	(rst = '1')  or (veto_en = '0') then
				goodtrig_q <= '0';
				veto_set   <= '0';
				
			else
				goodtrig_q 	<= goodtrig;
				veto_set 	<= '0';
				
				if  (goodtrig_q = '1') then
					if (p_match_count >= p_match_count_max) then
						veto_set <= '1';

					end if;
				end if;
			end if;
		end if;
	end process;


	-- Generater long veto signal
	--------------------------------------------------------
	veto_signal : process (clk)
	begin

		if	rising_edge(clk) then
			if	(rst = '1') or (veto_en = '0') then
				veto <= '0';
				vetolen_count 	<= (others => '0');

			elsif (veto_set = '1') then
					veto <= '1';
					vetolen_count <= (others => '0');

			elsif (vetolen_count = vetolen_count_max ) then
					veto <= '0';
					vetolen_count <= (others => '0');

			elsif (veto = '1') then
					vetolen_count <= vetolen_count + '1';

			else
				veto <= '0';
				vetolen_count <= (others => '0');

			end if;
		end if;
	end process;
	
	
	-- FFVeto Clocks Counter
	----------------------------------------------------------
	
	wr_reg_veto_count <= db_wr and ( ren_fftv_count_lo_i
											or ren_fftv_count_hi_i 
											or ren_fftv_count_ex_i);
	
	veto_counter : process (clk)
	begin

		if	rising_edge(clk) then
			if	(rst = '1') or (wr_reg_veto_count = '1') then
				veto_count <= (others => '0');
				
			else
				if (veto = '1') then 
					veto_count <= veto_count + '1';
				end if;
			
			end if;
		end if;
	end process;
	
	db_io <= veto_count(15 downto  0) when ((db_rd and ren_fftv_count_lo_i) = '1') else (others => 'Z');
	db_io <= veto_count(31 downto 16) when ((db_rd and ren_fftv_count_hi_i) = '1') else (others => 'Z');
	db_io <= veto_count(47 downto 32) when ((db_rd and ren_fftv_count_ex_i) = '1') else (others => 'Z');
				
	
	
	-- FV ID Counter
	----------------------------------------------------------
	
	rgw_fv_id <= db_wr and (ren_fv_id_hi_i or ren_fv_id_lo_i);
	
	fv_id_counter : process (clk)
	begin

		if	rising_edge(clk) then
			if	(rst = '1') or (rgw_fv_id = '1') then
				fv_id <= (others => '0');
				
			else
				if (veto_set = '1') then 
					fv_id <= fv_id + '1';
					
				end if;
			end if;
		end if;
	end process;
	
	db_io <= fv_id(31 downto 16) when ((db_rd and ren_fv_id_hi_i) = '1') else (others => 'Z');
	db_io <= fv_id(15 downto  0) when ((db_rd and ren_fv_id_lo_i) = '1') else (others => 'Z');

end rtl_tim;

-------------------------------------------------------------------------------
