-- hds header_start
--
-- VHDL Architecture TIM_fpga1.reset_gen.rtl
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 11:42:59 10/06/03
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

-- Reset Timer: 
-- Waits for other things (like VME interface to settle) before issuing reset


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity reset_gen is
   generic( 
      RST_LEN : integer := 255
   );
   port( 
      clk_in       : in     std_logic;
      reset1_i     : in     std_logic;
      reset2_i     : in     std_logic;
      reset3_i     : in     std_logic;
      dll_locked_i : in     std_logic;
      dll_reset_o  : out    std_logic;
      reset_o      : out    std_logic;
      init_i       : in     std_logic
   );

-- Declarations

end reset_gen ;


-- hds interface_end
architecture rtl of reset_gen is

	-- default values set here as there is no reset line
	
	signal reset_in   		: std_logic;
	signal reset_inq 			: std_logic_vector(1 downto 0);
	signal reset_in_rising  : std_logic;
	signal rst_timer  		: std_logic_vector(7 downto 0);
	signal rst_timer_max		: std_logic_vector(7 downto 0);
	signal rst_timer_en 		: std_logic := '0';
	
	signal init_q 				: std_logic_vector(1 downto 0);
	signal init_in 			: std_logic;
	
	
begin

	reset_in 		<= reset1_i or reset2_i or reset3_i;
	
	init_in 			<= init_i or init_q(0) or init_q(1);
	
	rst_timer_max 	<= conv_std_logic_vector(RST_LEN, 8);
	

	-------------------------------------------------------------
	reset_machine : process (clk_in)
		variable state : integer range 0 to 3 := 0;

	begin

		if rising_edge(clk_in) then
		
			-- make init nice and long!
			init_q(0) <= init_i;
			init_q(1) <= init_q(0);
			
			if (init_in = '1') then
				
				reset_inq 		 <= "00";
				reset_in_rising <= '0';
				rst_timer  		 <= "00000000";
				rst_timer_en 	 <= '0';

				
	 		else
	 		
				-- synchronise reset signal
				reset_inq(0) <= reset_in;
				reset_inq(1) <= reset_inq(0);
				reset_in_rising <= reset_inq(0) and not reset_inq(1);


		  		if (state = 0) then
			   	dll_reset_o   <= '1';
					rst_timer_en  <= '0';
	     			reset_o 		  <= '0';
		  			
	   			state := 1;
	   			
	   		elsif (state = 1) then
			   	dll_reset_o   <= '0';
					rst_timer_en  <= '1';
	
	   			reset_o <= rst_timer(7);
	   			   			
	   			if (rst_timer = rst_timer_max) then
	   				state := 2;
	   			else
	   				state := 1;
	   			end if;
	   			
	   		elsif (state = 2) then
					dll_reset_o   <= '0';
					rst_timer_en  <= '0';
		  			reset_o 		  <= '1';
	   			
	   			if (dll_locked_i = '1') then
	   				state := 3;
	   			else
	   				state := 2;
	   			end if;
	   		
	   		-- rest position
				elsif (state = 3) then
			   	dll_reset_o   <= '0';
					rst_timer_en  <= '0';
	     			reset_o 		  <= '0';
	     			
		  			if (reset_in_rising = '1') then
		  				state := 0;
		  			else
		  				state := 3;
		  			end if;
		  			
	   		end if;
	   		
	   	end if;
	   	
	   	-- Reset Timer Counter
	   	if (rst_timer_en = '1') then
	   		rst_timer <= rst_timer + '1';
	   	end if;
		end if;	
 	end process;

end rtl;
