-- hds header_start
--
-- VHDL Architecture TIM_fpga1.vme.untitled
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 13:56:45 02/04/04
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity vme is
   port( 
      clk            : in     std_logic;
      clk2x          : in     std_logic;
      rst            : in     std_logic;
      AM_i           : in     std_logic_vector (5 downto 0);
      AS_ni          : in     std_logic;
      BErr_ni        : in     std_logic;
      BaseAddr_Sel_i : in     std_logic;
      BaseAddr_i     : in     std_logic_vector (31 downto 16);
      DS0_ni         : in     std_logic;
      DS1_ni         : in     std_logic;
      GAP_ni         : in     std_logic;
      GA_ni          : in     std_logic_vector (4 downto 0);
      IAck_ni        : in     std_logic;
      IAckin_ni      : in     std_logic;
      LWord_ni       : in     std_logic;
      Write_ni       : in     std_logic;
      AB_i           : in     std_logic_vector (31 downto 0);
      Ack_o          : out    std_logic;
      ErrorLED_no    : out    std_logic;
      GAoutEn_o      : out    std_logic;
      IAckout_no     : out    std_logic;
      Irq_no         : out    std_logic_vector (7 downto 1);
      LE1_no         : out    std_logic;
      LE2_no         : out    std_logic;
      LRD_no         : out    std_logic;
      LWR_no         : out    std_logic;
      OEin_no        : out    std_logic;
      OEout_no       : out    std_logic;
      vmeBDSEL_o     : out    std_logic;
      vmeCLK_o       : out    std_logic;
      vmeRD_o        : out    std_logic;
      vmeWR_o        : out    std_logic;
      ab16_o         : out    std_logic
   );

-- Declarations

end vme ;


-- hds interface_end

architecture rtl of vme is

	signal Write			: std_logic;
	signal AS 	       	: std_logic;
	signal DS0				: std_logic;
	signal DS1				: std_logic;
	signal DS				: std_logic;
	signal GA           	: std_logic_vector( 4 downto 0);
	signal GAP    			: std_logic;
	signal GA_GAP     	: std_logic_vector( 5 downto 0);
	signal IAck   	      : std_logic;
--	signal IAckin 	      : std_logic;
	signal LWord  	      : std_logic;
	signal BErr				: std_logic;

	signal ga_mode	   : std_logic;
	signal ga_slot13  : std_logic;
	signal ga_error   : std_logic;


	signal am_base_ok		: std_logic;
	signal bdsel			: std_logic;
	signal addr         	: std_logic_vector(31 downto 0);

	signal ds_rising1    : std_logic;
	signal ds_rising2    : std_logic;
	signal ds_falling 	: std_logic;

	signal a24				: std_logic; 
	signal a32				: std_logic; 
                                     
	signal baseaddr24_ok	: std_logic; 
	signal baseaddr32_ok	: std_logic; 
	
	constant GA_BASE32 : std_logic_vector(15 downto 0) := "0000110100000000";
   constant GA_BASE24 : std_logic_vector( 7 downto 0) :=         "00001101";

begin

 	-- Negative Logic Conversions
	-----------------------------
	Write		<= not Write_ni;
	
	DS0		<= not DS0_ni;
	DS1		<= not DS1_ni;
	AS 		<= not AS_ni;
	BErr 		<= not BErr_ni;
	GA     	<= not GA_ni;
	GAP    	<= not GAP_ni;
	IAck   	<= not IAck_ni;
--	IAckin 	<= not IAckin_ni;
	LWord		<= not LWord_ni;


	---------------------------------------------------------------------------
	-- Address Setup Block
	---------------------------------------------------------------------------
	address_blk : block

		signal IAck_AM		: std_logic_vector ( 6 downto 0);
		
		signal bdaddr32	: std_logic_vector (31 downto 16);
		signal bdaddr24	: std_logic_vector (23 downto 16);
		
		signal asq			: std_logic_vector ( 2 downto  0);
	
	---------------------------------------------------------------------------
	begin


		-- *** note: these sigs are all async, but are only used after DS is sync'd
		-- this should ensure no problems!, but how do we tell the simulator this
		
		GA_GAP 	<= (GA & GAP);
		IAck_AM	<= (IAck & AM_i(5 downto 0)); 
		

		ga_mode 	 <= BaseAddr_Sel_i; -- zero = switch set addr
		
		ga_slot13 <= '1' when (GA_GAP = "011010") else '0';
		ga_error  <= '0' when (GA_GAP = "011010") or (GA_GAP = "000000") else '1';
		

		a24 <= '1' when (IAck_AM = "0111001") or (IAck_AM = "0111101") else '0';
		a32 <= '1' when (IAck_AM = "0001001") or (IAck_AM = "0001101") else '0';

		
		bdaddr32 <= GA_BASE32 when (ga_mode = '1') else BaseAddr_i(31 downto 16);
		bdaddr24 <= GA_BASE24 when (ga_mode = '1') else BaseAddr_i(23 downto 16);

		baseaddr32_ok <= '1' when (ab_i(31 downto 16) = bdaddr32) else '0';
		baseaddr24_ok <= '1' when (ab_i(23 downto 16) = bdaddr24) else '0';
		
		am_base_ok	<= (a24 and baseaddr24_ok) or (a32 and baseaddr32_ok);
		GAoutEn_o 	<= ga_slot13;
		
		
		-- Board Select Latch
		----------------------------------------------------------------
		
		process (clk)
		begin
			
			if rising_edge(clk) then
				if (rst = '1') then
					bdsel <= '0';
					
				else
					if (ds_rising1 = '1') and (am_base_ok = '1') then
						bdsel <= '1';
				
					elsif (ds_falling = '1') then
						bdsel <= '0';
				
					end if;
				end if;
			end if;
		end process;
		

		-- Address Bus Latch
		------------------------------------------------------------------------
	 	addr(0) <= '0';

		process (clk)
		begin

			if rising_edge(clk) then
				if (rst = '1') then
					addr(31 downto 1) <= (others => '0');
					asq  <= (others => '0');
				
				else
					asq <= (asq(1 downto 0) & AS); -- includes sync
								
					if (as_rising = "10") then --rising
			   		addr(31 downto 1) <= ab_i(31 downto 1);
			   	
			   	end if;
			   end if;
		 	end if;
		end process;
		
		as_rising <= asq(1) and (not asq(2));
		ab16_o <= addr(15 downto 0);
		
	end block;
	---------------------------------------------------------------------------


	---------------------------------------------------------------------------
	-- VME IO Control
	---------------------------------------------------------------------------
	io_control : block

		signal dsq			: std_logic_vector(5 downto 0);
                        
		signal wrq0			: std_logic;
		signal rdq0			: std_logic;

		signal vmebdsel	: std_logic;
		signal vmewr		: std_logic;
		signal vmerd 		: std_logic;
		signal le1			: std_logic;
		signal le2    		: std_logic;
		signal Ack  		: std_logic;

	begin

		-- DS synchroniser
		-------------------------
		-- *** note internal delay generating pipeling -- ***25ns steps
		process (clk)
		begin

			if rising_edge(clk) then
				if (rst = '1') then
					DS  <= '0';
					dsq <= (others => '0');
			
				else
					DS	 <= DS0 or DS1; -- *** added seperately for better sync		
											 --     should be included in dsq chain later
					dsq <= ( dsq(4 downto 0) & DS );

				end if;
			end if;
		end process;

		ds_rising1 <= dsq(0) and (not dsq(1));
		ds_rising2 <= dsq(1) and (not dsq(2));
		ds_falling <= dsq(1) and (not dsq(0));


		-- I/O Control --
		------------------
		process (clk)
		begin
			if rising_edge(clk) then
				if (rst = '1') then
					wrq0 		<= '0';
					rdq0 		<= '0';
					vmebdsel <= '0';
					vmewr 	<= '0';
					vmerd 	<= '0';
					le1		<= '0';
					le2 		<= '0';
					ack   	<= '0';

				else
					wrq0 		<= Write;
					rdq0 		<= not Write;
					vmewr 	<= bdsel and wrq0;
					vmerd 	<= bdsel and rdq0;
					le1 		<= not (bdsel and dsq(4));
					le2 		<= bdsel and dsq(4) and wrq0;
					ack 		<= bdsel and dsq(5);

				end if;
			end if;
		end process;


		vmeBDSEL_o <= bdsel;
		vmeWR_o    <= vmewr;
		vmeRD_o    <= vmerd;
		vmeCLK_o   <= le2; 

		LRD_no	  <= not vmerd; -- old bRead but goes to latches only
		LWR_no	  <= not vmewr; -- old bWrite   ditto
		OEout_no   <= not vmerd;
		OEin_no    <= not vmewr;
		
		LE1_no     <= not le1;
		LE2_no     <= not le2;
		
		Ack_o	     <= ack;

	end block;

	ErrorLED_no <= not (BErr or (ga_error and ga_mode));

	---------------------------------------------------------------------------
	-- IRQ Stuff - disabled for now
	---------------------------------------------------------------------------

  	Irq_no     <= "1111111";
  	IAckout_no <= IAckin_ni;

end rtl;
