-- hds header_start
--
-- VHDL Architecture TIM_fpga2.rod_busys.rtl2
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 14:12:08 03/22/04
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity rod_busys is
   port( 
      rst                 : in     std_logic;
      clk                 : in     std_logic;
      runmode_i           : in     std_logic;
      db                  : inout  std_logic_vector (15 downto 0);
      db_rd               : in     std_logic;
      db_wr               : in     std_logic;
      reg_rb_mask_i       : in     std_logic_vector (15 downto 0);
      Reg20               : in     std_logic;
      Reg22               : in     std_logic;
      Reg24               : in     std_logic;
      ren_busy_count_lo_i : in     std_logic;
      ren_busy_count_hi_i : in     std_logic;
      ren_busy_count_ex_i : in     std_logic;
      bp_RODBUSY_i        : in     std_logic_vector (15 downto 0);
      v_RodBusy_i         : in     std_logic;
      ext_RODBUSYin_i     : in     std_logic;
      ext_RODBUSYinen_i   : in     std_logic;
      rodcrate_busy_o     : out    std_logic;
      rodbusy_o           : out    std_logic;
      ext_rodbusy_o       : out    std_logic;
      sarb_disable_i      : in     std_logic;
      busy_clkswitch_i    : in     std_logic;
      fftveto_i           : in     std_logic;
      led_rodbusy_o       : out    std_logic_vector (15 downto 0)
   );

-- Declarations

end rod_busys ;


-- hds interface_end

architecture rtl of rod_busys is

	signal RegRB_Latch   		: std_logic_vector(15 downto 0);
	signal wr_reg_rb_latch		: std_logic;

	signal RegRB_Monitor			: std_logic_vector(15 downto 0);

	signal rb_store      		: std_logic_vector(15 downto 0);
	signal rb_store_16q 			: std_logic_vector(15 downto 0);
	signal count_div16  			: std_logic_vector(3 downto 0);
	
	signal busy_count				: std_logic_vector(47 downto 0);
	signal wr_reg_busy_count	: std_logic;
	
	
	signal rodcrate_busy					: std_logic;
	signal ext_rodbusy					: std_logic;
	
	signal rodbusy_q0				: std_logic_vector(15 downto 0);
	signal rodbusy_q				: std_logic_vector(15 downto 0);
	
	
	signal rodbusy      		: std_logic;
	
	
	
	

begin

	-----------------------------------------------------------------------------
	-- Rod Busy Sync
	-----------------------------------------------------------------------------
	process (clk)
	begin
		if rising_edge(clk) then
			if (rst = '1') then
				rodbusy_q0	<= (others => '0');
				rodbusy_q  	<= (others => '0');		
				
			else
				rodbusy_q0	<= bp_RODBusy_i;
				rodbusy_q  	<= rodbusy_q0;

			end if;
		end if;
	end process;


	-----------------------------------------------------------------------------
	-- Rod Busy LEDs
	-----------------------------------------------------------------------------
	led_rodbusy_o <= rodbusy_q or rodbusy_q0;


	-----------------------------------------------------------------------------
	-- RegRB_Latch - one way latch - stores busys - reset by write
	-----------------------------------------------------------------------------

	wr_reg_rb_latch <= db_wr and Reg22;

	process (clk)
	begin
		if rising_edge(clk) then
			if (rst = '1') or (wr_reg_rb_latch = '1') then
				RegRB_Latch <= (others => '0');

			else
				RegRB_Latch <= rodbusy_q or RegRB_Latch;

			end if;
		end if;
	end process;



   ----------------------------------------------------------------------------
   -- RB_Monitor - Latches busy if high for 16 to 31 clocks	-- reset by write     
   ----------------------------------------------------------------------------

	-- *** Not too sure if this works ...
	process (clk)
	begin
		if rising_edge(clk) then
			if (rst = '1') then			
				rb_store		 <= (others => '0');
				rb_store_16q <= (others => '0');
				count_div16  <= (others => '1');

			else
				if (count_div16 = "0000") then
					rb_store <= (others => '1');
					rb_store_16q <= rb_store;
					
					count_div16  <= (others => '1');

				else
					rb_store 	 <= rodbusy_q and rb_store;
					count_div16  <= count_div16 - '1';
				
				end if;
        	end if;
		end if;
	end process;


	process (clk)
	begin
		if rising_edge(clk) then
			if (rst = '1') then
				RegRB_Monitor <= (others => '0');
	
			else
				if (Reg24 and db_wr) = '1' then
					RegRB_Monitor <= (others => '0');
	
				else -- self holding
					RegRB_Monitor <= rb_store_16q or RegRB_Monitor;
	
				end if;
			end if;
		end if;
	end process;
	
   ----------------------------------------------------------------------------
   -- Rod(Crate)Busy Outs
   ----------------------------------------------------------------------------
           
	rodbusy <= '1' when ((rodbusy_q AND reg_rb_mask_i) /= "0000000000000000") 
						else '0';
	
	ext_rodbusy  <= (ext_RodBUSYin_i and ext_RodBUSYinen_i);
              
	rodcrate_busy <=		rodbusy
				 			or v_RodBusy_i
				 			or (not(runmode_i) and not(sarb_disable_i))
				 			or busy_clkswitch_i
				 			or fftveto_i;	 

	rodcrate_busy_o <= rodcrate_busy;
	rodbusy_o 		 <= rodbusy;
	ext_rodbusy_o 	 <= ext_rodbusy;


	-- Busy Counter
	----------------------------------------------------------
	
	wr_reg_busy_count <= db_wr and ( 	ren_busy_count_lo_i 
												or ren_busy_count_hi_i 
										   	or ren_busy_count_ex_i	);
	
	busy_counter : process (clk)
	begin

		if	rising_edge(clk) then
			if	(rst = '1') or (wr_reg_busy_count = '1') then
				busy_count <= (others => '0');
				
			else
				if (rodcrate_busy = '1') then 
					busy_count <= busy_count + '1';
				end if;
			
			end if;
		end if;
	end process;
	


  -----------------------------------------------------------------------------
  -- Data Bus
  -----------------------------------------------------------------------------

	db <= rodbusy_q  					 when ((Reg20 					 and db_rd) = '1') else (others => 'Z');
	db <= RegRB_Latch 				 when ((Reg22 					 and db_rd) = '1') else (others => 'Z');
	db <= RegRB_Monitor 				 when ((Reg24 					 and db_rd) = '1') else (others => 'Z');
	db <= busy_count(15 downto  0) when ((ren_busy_count_lo_i and db_rd) = '1') else (others => 'Z');
	db <= busy_count(31 downto 16) when ((ren_busy_count_hi_i and db_rd) = '1') else (others => 'Z');
	db <= busy_count(47 downto 32) when ((ren_busy_count_ex_i and db_rd) = '1') else (others => 'Z');
		   --(others => 'Z');

end rtl;
