-- hds header_start
--
-- VHDL Architecture TIM_fpga2.serialser.rtl
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 14:25:38 02/03/04
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity ser_control is
   port( 
      rst             : in     std_logic;
      clk             : in     std_logic;
      db              : inout  std_logic_vector (15 downto 0);
      db_rd           : in     std_logic;
      db_wr           : in     std_logic;
      ren_fifostat_i  : in     std_logic;
      EX_Strobe_i     : in     std_logic;
      EXID_i          : in     std_logic_vector (35 downto 0);
      SerEXS_o        : out    std_logic;
      TT_Strobe_i     : in     std_logic;
      TTID_i          : in     std_logic_vector (9 downto 0);
      SerTTS_o        : out    std_logic;
      exf_read_o      : out    std_logic;
      exf_write_o     : out    std_logic;
      exf_data_o      : out    std_logic_vector (35 downto 0);
      exf_data_i      : in     std_logic_vector (35 downto 0);
      exf_full_i      : in     std_logic;
      exf_empty_i     : in     std_logic;
      exf_depth_i     : in     std_logic_vector (5 downto 0);
      ttf_read_o      : out    std_logic;
      ttf_write_o     : out    std_logic;
      ttf_data_o      : out    std_logic_vector (9 downto 0);
      ttf_data_i      : in     std_logic_vector (9 downto 0);
      ttf_full_i      : in     std_logic;
      ttf_empty_i     : in     std_logic;
      ttf_depth_i     : in     std_logic_vector (5 downto 0);
      ren_tts_count_i : in     std_logic
   );

-- Declarations

end ser_control ;


-- hds interface_end
architecture rtl of ser_control is

   signal RegFIFOStat   : std_logic_vector(15 downto 0);

   
begin


-------------------------------------------------------------------------------
-- MAIN CODE
-------------------------------------------------------------------------------
         
   -----------------------------------------------------------------      
	-- Extended ID 
   -----------------------------------------------------------------
   exs: block
 		
  		signal exs_busy		: std_logic;
   	signal exs_out       : std_logic;
   	
   	CONSTANT IDLE 			: integer := 61;
   	CONSTANT READ_FIFO	: integer := 62;
   	CONSTANT START_BIT	: integer := 63;
   	CONSTANT SERIALISE	: integer := 0;
   	
      
   begin
    

   	exf_data_o   <= EXID_i;
      exf_write_o  <= EX_Strobe_i;

		---------------------------------------------------------------

   	ev_state_machine: process (clk)
   	
   	  	variable exsm_state : integer range 0 to 63;
   		   	
   	begin
   	
   		if rising_edge(clk) then 
   			if (rst = '1') then
   				exsm_state := 61;
  				  	exs_out	  <= '0';
					exf_read_o <= '0';
   			
   			else

				   --defaults
				   exs_out	  <= '0';
				   exf_read_o <= '0';			   	
			
					-- idle
				   if (exsm_state = IDLE) then
						
						if (exf_empty_i = '0') then
			   				exsm_state := exsm_state + 1;
			   		end if;
			   		
			
			   	-- start 0
			   	elsif (exsm_state = READ_FIFO) then
					   exf_read_o	<= '1'; -- Sync FIFO outputs data on NEXT clk
			   		
			   		exsm_state := exsm_state + 1;
			

			   	-- start 1
			   	elsif (exsm_state = START_BIT) then
					   exs_out <= '1'; --start bit
			   		
			   		--exsm_state := exsm_state + 1; -- this should roll to 0
						exsm_state := SERIALISE;
			   		
			   	-- end
			   	elsif (exsm_state = 36) then
			   		exs_out	<= '0'; -- stop bit
			   		
			   		exsm_state := IDLE;
			   		
			   		
					-- serialise is default option
			   	else
			   		exs_out <= exf_data_i(exsm_state);
			   	
			   		exsm_state := exsm_state + 1;
			   	
			   	end if;
			   end if;
			end if;
		end process;
	   
		SerEXS_o <= exs_out;

   end block;				
       	


   -----------------------------------------------------------------      
	-- Trigger Type 
   -----------------------------------------------------------------
	tts: block
 		
  		signal tts_busy		: std_logic;
   	signal tts_out       : std_logic;
   	signal tts_out_q     : std_logic;
   	
   	signal tts_count		: std_logic_vector(15 downto 0);
   	   	
   	CONSTANT IDLE 			: integer := 13;
   	CONSTANT READ_FIFO	: integer := 14;
   	CONSTANT START_BIT	: integer := 15;
   	CONSTANT SERIALISE	: integer := 0;
   	
      
   begin

   	ttf_data_o   <= ttID_i;
      ttf_write_o  <= tt_Strobe_i;

		---------------------------------------------------------------

   	tt_state_machine: process (clk)
   	
   	  	variable ttsm_state : integer range 0 to 15;
   		   	
   	begin
   	
   		if rising_edge(clk) then 
   			if (rst = '1') then
   				ttsm_state := IDLE;
  				  	tts_out	  <= '0';
					ttf_read_o <= '0';
   			
   			else

				   --defaults
				   tts_out	  <= '0';
				   ttf_read_o <= '0';			   	
			
					-- idle
				   if (ttsm_state = IDLE) then
						
						if (ttf_empty_i = '0') then
			   				ttsm_state := ttsm_state + 1;
			   		end if;
			   		
			
			   	-- start 0
			   	elsif (ttsm_state = READ_FIFO) then
					   ttf_read_o	<= '1'; -- Sync FIFO outputs data on NEXT clk
			   		
			   		ttsm_state := ttsm_state + 1;
			

			   	-- start 1
			   	elsif (ttsm_state = START_BIT) then
					   tts_out <= '1'; --start bit
			   		
			   		--ttsm_state := ttsm_state + 1; -- this should roll to 0
						ttsm_state := SERIALISE;
			   		
			   	-- end
			   	elsif (ttsm_state = 10) then
			   		tts_out	<= '0'; -- stop bit
			   		
			   		ttsm_state := IDLE;
			   		
			   		
					-- serialise is default option
			   	else
			   		tts_out <= ttf_data_i(ttsm_state);
			   	
			   		ttsm_state := ttsm_state + 1;
			   	
			   	end if;
			   end if;
			end if;
		end process;
	   
		SerTTS_o <= tts_out;
		
		
	-- TType Count register - counts number of TType strobes *** DOESN'T LOOK LIKE IT!
   -------------------------------------------------------------------------
   process (clk)
   begin

      if rising_edge(clk) then

         if (rst = '1') then
         	tts_count <= (others => '0');
         	tts_out_q <= '0';

			else
				tts_out_q <= tts_out;
			
         	if ((ren_tts_count_i and db_wr )= '1') then
         		tts_count <= (others => '0');
         		
         	elsif (tts_out = '1') and (tts_out_q = '0') then --rising
   				tts_count <= tts_count + '1';
   
            end if;
         end if;
      end if;
   end process;
   
	db <= tts_count when ((ren_tts_count_i and db_rd) = '1') else (others => 'Z');
	
		

   end block;								
	---------------------------------------------------------------------------

   
         
         
  	-- FIFO Status Register - 0x0E
  	----------------------------------------------------
         
	RegFIFOStat(15) 				<= ttf_full_i;
	RegFIFOStat(14) 				<= ttf_empty_i;
	RegFIFOStat(13 downto 8) 	<= ttf_depth_i;
	
	RegFIFOStat(7) 				<= exf_full_i;
	RegFIFOStat(6) 				<= exf_empty_i;
	RegFIFOStat(5 downto 0) 	<= exf_depth_i;
	
	db <= RegFIFOStat when ((db_rd and ren_fifostat_i) = '1') else (others => 'Z');

  
end rtl;
