-- hds header_start
--
-- VHDL Entity TIM_fpga2.serialser.symbol
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 14:14:48 02/03/04
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity serialser is
   port( 
      -- Common Signals
            -----------------
      reset       : in     std_logic;                       -- was NOT nOA_rst
      clk         : in     std_logic;
      --RunMode        : in  std_logic;
      db          : inout  std_logic_vector (15 downto 0);
      vmeRD       : in     std_logic;                       -- was bReadb
      vmeWR       : in     std_logic;                       -- was bWriteb
      --vmeCLK       	: in  std_logic;   -- was LEb2 (for clockless registers)
      Reg0E       : in     std_logic;
      -- Serialser Signals
            --------------------
      EV_Strobe_i : in     std_logic;
      EVID_i      : in     std_logic_vector (43 downto 0);
      SerEVS_o    : out    std_logic;
      TT_Strobe_i : in     std_logic;
      TTID_i      : in     std_logic_vector (9 downto 0);
      SerTTS_o    : out    std_logic;
      -- Debug Bus     
            ----------------
      DebugBus    : out    std_logic_vector (15 downto 0)
   );

-- Declarations

end serialser ;

-- hds interface_end

------------------------------------------------------------------------------
-- ARCHITECTURE                                                                
------------------------------------------------------------------------------
architecture tim of serialser is  
	
	component cg_fifo_64x10
		port (
			clk			: in 	std_logic;
			sinit			: in 	std_logic;
			din			: in 	std_logic_vector(9 downto 0);
			wr_en			: in 	std_logic;
			rd_en			: in 	std_logic;
			dout			: out std_logic_vector(9 downto 0);
			full			: out std_logic;
			empty			: out std_logic;
			data_count	: out std_logic_vector(5 downto 0)
		);
	end component;
	
	
	component cg_fifo_64x44
		port (
			clk			: in  std_logic;
			sinit			: in  std_logic;
			din			: in  std_logic_vector(43 downto 0);
			wr_en			: in  std_logic;
			rd_en			: in  std_logic;
			dout			: out std_logic_vector(43 downto 0);
			full			: out std_logic;
			empty			: out std_logic;
			data_count	: out std_logic_vector(5 downto 0)
		);
	end component; 


------------------------------------------------------------------------------
-- SIGNAL DECLARATIONS
------------------------------------------------------------------------------

	type ser_state is (ready, serialise); 		

   signal RegFIFOStat   : std_logic_vector(15 downto 0);
   
	signal evf_read   	: std_logic;
	signal evf_write  	: std_logic;
	signal evf_datain 	: std_logic_vector(43 downto 0);
	signal evf_dataout	: std_logic_vector(43 downto 0);
	signal evf_full   	: std_logic;
	signal evf_empty  	: std_logic;
	signal evf_depth  	: std_logic_vector(5 downto 0);

	signal ttf_read   	: std_logic;
	signal ttf_write  	: std_logic;
	signal ttf_datain 	: std_logic_vector(9 downto 0);
	signal ttf_dataout	: std_logic_vector(9 downto 0);
	signal ttf_full   	: std_logic;
	signal ttf_empty  	: std_logic;
	signal ttf_depth  	: std_logic_vector(5 downto 0);
	
	--signal tts_busy		: std_logic;

   
begin

-------------------------------------------------------------------------------
-- INSTANTIATIONS
-------------------------------------------------------------------------------   
	UID_FIFO: cg_fifo_64x44
		port map (
			clk			=>	clk,
			sinit			=>	reset,
			din			=> evf_datain,
			wr_en			=> evf_write,
			rd_en			=>	evf_read,
			dout			=> evf_dataout,
			full			=> evf_full,
			empty			=> evf_empty,
			data_count	=> evf_depth
		);
		
	UTT_FIFO: cg_fifo_64x10
		port map (
			clk			=>	clk,
			sinit			=>	reset,
			din			=>	ttf_datain,
			wr_en			=> ttf_write,
			rd_en			=> ttf_read,
			dout			=> ttf_dataout,
			full			=> ttf_full,
			empty			=> ttf_empty,
			data_count	=> ttf_depth
		);


-------------------------------------------------------------------------------
-- MAIN CODE
-------------------------------------------------------------------------------
         
   -----------------------------------------------------------------      
	-- Event ID 
   -----------------------------------------------------------------
   evs: block

   	signal state			: ser_state;
   	signal next_state		: ser_state;
  		
  		signal evs_busy		: std_logic;
   	signal evs_data      : std_logic_vector(43 downto 0);
   	signal evs_out       : std_logic;
   	signal evs_count     : std_logic_vector( 5 downto 0);
      
   begin

   	evf_datain <= EVID_i;
      evf_write  <= EV_Strobe_i;

		---------------------------------------------------------------
   	evs_control: process (state, evf_empty, evs_count)
	  	begin
  
     		case state is
   			when ready =>
   				evs_busy <= '0';
   				if (evf_empty='0') then
   					next_state <= serialise;
   				else
   					next_state <= ready;
   				end if;
   				
   			when serialise =>
   				evs_busy <= '1';
   				if (evs_count = "100100") then --36d
   					next_state <= ready; 
   				else
   					next_state <= serialise;
   				end if;
   			
   		end case;
   	end process;
   			
		SerEVS_o <= evs_out;
   	   	
		---------------------------------------------------------------
   	sync_state: process (reset, clk)
   	begin

   		if reset = '1' then
				evf_read		<= '0';
   			evs_data    <= (others => '0');
				state   		<= ready;
				evs_out     <= '0';
				evs_count	<= "000000";
				
   		elsif rising_edge(clk) then
   		
				state <= next_state;
   			
   			if (evs_busy = '1') then
  					evf_read		<= '0';
				  	evs_out 	<= evs_data(0);
		   		evs_data(42 downto 0) <= evs_data(43 downto 1);
		   		evs_count <= evs_count + '1';

  				else
  					evf_read		<= '1';
	   			evs_data 	<= evf_dataout(43 downto 0);
	   			evs_out 		<= '0';
	   			evs_count 	<= "000000";
	   			
	   		end if;
	   	end if;
	   end process;


   end block;				
       	


   -----------------------------------------------------------------      
	-- Trigger Type 
   -----------------------------------------------------------------
   tts: block
   	signal state			: ser_state;
   	signal next_state		: ser_state;
   	
  		signal tts_busy		: std_logic;
   	signal tts_data      : std_logic_vector(9 downto 0);
   	signal tts_out       : std_logic;
   	signal tts_count     : std_logic_vector( 3 downto 0);
   	
   begin
     	     	
         	
      ttf_datain <= TTID_i;
      ttf_write  <= TT_Strobe_i;


   	--------------------------------------------------------------      
   	tts_control: process (state,  ttf_empty, tts_count)
	  	begin
  
     		case state is
   			when ready =>
   				tts_busy <= '0';
   				if (ttf_empty='0') then
   					next_state <= serialise;
   				else
   					next_state <= ready;
   				end if;
   				
   			when serialise =>
   				tts_busy <= '1';
   				if (tts_count = "1001") then --9d
   					next_state <= ready; 
   				else
   					next_state <= serialise;
   				end if;
   			
   		end case;
   	end process;

		--------------------------------------------------------------
   	sync_state: process (reset, clk)
   	begin

   		if reset = '1' then
				ttf_read		<= '0';
   			tts_data    <= (others => '0');
				state   		<= ready;
				tts_out     <= '0';
				tts_count	<= "0000";
				
   		elsif rising_edge(clk) then
   			state <= next_state;

   			if (tts_busy = '1') then
  					ttf_read		<= '0';
				  	tts_out 		<= tts_data(0);
		   		tts_count 	<= tts_count + '1';
		   		tts_data(8 downto 0) <= tts_data(9 downto 1);

  				else
  					ttf_read		<= '1';
	   			tts_out 		<= '0';
	   			tts_count 	<= "0000";
	   			tts_data 	<= ttf_dataout(9 downto 0);
	   			
	   		end if;
	   	end if;
	   	
	   end process;
   	
		SerTTS_o <= tts_out; 
   			
   end block;				
	---------------------------------------------------------------------------

   
         
         
  	-- FIFO Status Register - Reg0E
  	----------------------------------------------------
         
	RegFIFOStat(15) 				<= ttf_full;
	RegFIFOStat(14) 				<= ttf_empty;
	RegFIFOStat(13 downto 8) 	<= ttf_depth;
	
	RegFIFOStat(7) 				<= evf_full;
	RegFIFOStat(6) 				<= evf_empty;
	RegFIFOStat(5 downto 0) 	<= evf_depth;



  	-----------------------------------------------------------------------------
  	-- Data Bus
  	-----------------------------------------------------------------------------
   
   db <= RegFIFOStat when ((vmeRD and Reg0E) = '1') else (others => 'Z');
   
   
   -- Debug Bus
  	----------------------------------------------------

	DebugBus <= "0000000000000000";

  
end tim;
