-- hds header_start
--
-- VHDL Architecture TIM_fpga2.serialser.rtl
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 14:25:38 02/03/04
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity tmon_control is
   port( 
      rst            : in     std_logic;
      clk            : in     std_logic;
      db             : inout  std_logic_vector (15 downto 0);
      db_rd          : in     std_logic;
      db_wr          : in     std_logic;
      dbh_io         : inout  std_logic_vector (15 downto 0);
      db_d32         : in     std_logic;
      ren_tp_fifol_i : in     std_logic;
      fl_read_o      : out    std_logic;
      fl_write_o     : out    std_logic;
      fl_data_o      : out    std_logic_vector (15 downto 0);
      fl_data_i      : in     std_logic_vector (15 downto 0);
      fl_full_i      : in     std_logic;
      fl_empty_i     : in     std_logic;
      fh_read_o      : out    std_logic;
      fh_write_o     : out    std_logic;
      fifo_rst_o     : out    std_logic;
      fh_data_o      : out    std_logic_vector (13 downto 0);
      fh_data_i      : in     std_logic_vector (13 downto 0);
      fh_full_i      : in     std_logic;
      fh_empty_i     : in     std_logic;
      ren_tp_fifoh_i : in     std_logic;
      trigger_i      : in     std_logic;
      fftveto_i      : in     std_logic;
      db_rd0         : in     std_logic
   );

-- Declarations

end tmon_control ;


-- hds interface_end
architecture rtl of tmon_control is

   signal tp_count : std_logic_vector(28 downto 0);
   signal fh_full  : std_logic;
	signal fh_empty : std_logic;
	signal fl_full  : std_logic;
	signal fl_empty : std_logic;
	
	signal fh_dbdata : std_logic_vector(15 downto 0);

----------------------------------------------------------------------------
-- MAIN CODE
----------------------------------------------------------------------------
begin
         
	trig_period_counter : process (clk)
	begin
	
		if rising_edge(clk) then 
			if (rst = '1') then
				tp_count <= (others => '0');
			  	
			else
				if (trigger_i = '1') then 
					tp_count <= (others => '0');
					
				elsif (tp_count = "11111111111111111111111111111") then  -- stack up in this time-bin
		   		tp_count <= "11111111111111111111111111111";  
		   		
		   	else
		   		tp_count <= tp_count + '1';
				
		   	end if;
		   end if;
		end if;
	end process;
   



  	-----------------------------------------------------------------------------
  	-- Fifo managers
  	-----------------------------------------------------------------------------
   
   -- FIFO's reset by write to either
   fifo_rst_o <= rst or (db_wr and (ren_tp_fifoh_i or ren_tp_fifol_i));
   
   
   -- FIFO Latch (hi is used for both stats) 
   -----------------------------------------
   fh_stat_latch : process (clk)
   begin
		if rising_edge(clk) then 
			if (rst = '1') then
				fh_full 	<= '0';
				fh_empty <= '0';
			  	
			else
				if (db_rd0 and (ren_tp_fifoh_i or (db_d32 and ren_tp_fifol_i))) = '1' then 
					fh_full 	<= fh_full_i ;
					fh_empty	<= fh_empty_i;
					
		   	end if;
		   end if;
		end if;
	end process;
	
	
	-- FIFO Hi
   ----------
      
   fh_data_o  <= (fftveto_i & tp_count(28 downto 16));
   fh_write_o <= trigger_i;
   fh_read_o  <= db_rd0 and (ren_tp_fifoh_i or (db_d32 and ren_tp_fifol_i));
   
   fh_dbdata <= (fh_full & fh_empty & fh_data_i);
   
   db 		 <= fh_dbdata when ((db_rd and ren_tp_fifoh_i) = '1') else (others => 'Z');
            
   -- FIFO Lo
   -----------
	fl_data_o  <= tp_count(15 downto 0);      
   fl_write_o <= trigger_i;                  
   fl_read_o  <= (db_rd0 and ren_tp_fifol_i);
      
   db 	 <= fl_data_i when ((db_rd and ren_tp_fifol_i) = '1')				  else (others => 'Z');
  
   -- D32
  	dbh_io <= fh_dbdata when ((db_rd and ren_tp_fifol_i and db_d32) = '1') else (others => 'Z');
  
  
end rtl;


-- FIFO Bits
-----------------------
-- Note: must read both fifo registers each time, (fifo_lo first ?)


-- FIFO Hi
--		15: FIFO fulls
--		14: FIFO empty
--		13: FFTV state at trigger
--(12:0): period(28:16)

-- FIFO Lo
--(15:0):  period(15:0)

