-- hds header_start
--
-- VHDL Architecture fpga2.exttid.rtl
--
-- Created:
--          by - warren (ELECTRON)
--          at - 15:44:17 03/23/04
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


-- *** log start
-- 2007-03-24  Added extra delay to bcid capture sig when sa_ttcl1a set
-- 2007-03-28  Added 1 more clock delay to bcid capture sig when sa_ttcl1a set
--             delayed strobe to fifos too
--             these may compensate for bp FFs
-- 2007-03-28  Added sa_ttcttid_en to get TTC TTID in SA mode. strobe too.
-- 2007-08-31  Added bcid_max_en to allow BCIDs to rollover at 0xFFF
-- *** log end
entity exttid is
   generic( 
      BCID_ROLLOVER : integer := 3564
   );
   port( 
      rst             : in     std_logic;
      clk             : in     std_logic;
      runmode         : in     std_logic;
      db              : inout  std_logic_vector (15 downto 0);
      db_rd           : in     std_logic;
      db_wr           : in     std_logic;
      dbh_io          : inout  std_logic_vector (15 downto 0);
      db_d32          : in     std_logic;
      Reg10           : in     std_logic;
      Reg12           : in     std_logic;
      Reg14           : in     std_logic;
      Reg16           : in     std_logic;
      Reg2A           : in     std_logic;
      TTCL1ID_i       : in     std_logic_vector (23 downto 0);
      TTCL1ID_strb_i  : in     std_logic;
      TTCBCID_i       : in     std_logic_vector (11 downto 0);
      TTCBCID_strb_i  : in     std_logic;
      TTCTTID_i       : in     std_logic_vector (9 downto 0);
      TTCTTID_strb_i  : in     std_logic;
      AllL1A_i        : in     std_logic;
      AllBCR_i        : in     std_logic;
      AllECR_i        : in     std_logic;
      EXID_o          : out    std_logic_vector (35 downto 0);
      EX_Strobe_o     : out    std_logic;
      TTID_o          : out    std_logic_vector (9 downto 0);
      TT_Strobe_o     : out    std_logic;
      enSABCR2_i      : in     std_logic;
      SABCR2_o        : out    std_logic;
      enSAECR2_i      : in     std_logic;
      SAECR2_o        : out    std_logic;
      TTCL1A_i        : in     std_logic;
      l1idro_ecr_en_i : in     std_logic;
      sa_ttcl1a_en_i  : in     std_logic;
      bcid_max_en_i   : in     std_logic;
      sa_ttcttid_en_i : in     std_logic
   );

-- Declarations

end exttid ;

-- hds interface_end

architecture rtl of exttid is


		signal AllL1A_q      : std_logic;
		signal AllL1A_q2     : std_logic;
		
		signal L1ID          : std_logic_vector(23 downto 0);
		signal BCID          : std_logic_vector(11 downto 0);
		signal TTID          : std_logic_vector( 9 downto 0);
	   signal ECRID         : std_logic_vector( 7 downto 0);
	   
  		signal RegL1ID 	   : std_logic_vector(23 downto 0);
  		signal RegBCID 	   : std_logic_vector(11 downto 0);
  		signal RegTTCBCID    : std_logic_vector(11 downto 0);
  		signal RegBCIDoffset : std_logic_vector( 3 downto 0);
      
		

------------------------------------------------------------------------------
-- MAIN CODE
------------------------------------------------------------------------------
begin

	------------------------------------------------------------------------
  	-- Global
	------------------------------------------------------------------------
	
	process (rst, clk)
	begin
	   if rising_edge(clk) then 
			if (rst = '1') then 
				AllL1A_q <= '0';
				AllL1A_q2 <= '0';
				
			else			
				AllL1A_q <= AllL1A_i;
				AllL1A_q2 <= AllL1A_q;
		   
		   end if;
		end if;
	end process;

	   	
   -- Outputs to Serialiser/Fifo
	EXID_o <= (BCID & L1ID);
	TTID_o <= TTID;
	
	
	---------------------------------------------------------------------------
	-- L1ID BLOCK
	---------------------------------------------------------------------------
	l1id_blk : block
	
		-- Strobes
		signal TTCL1ID_strb_q	: std_logic_vector( 2 downto 0);
		
		-- L1ID register and counter
		signal CountL1ID 			: std_logic_vector(23 downto 0);
	
		signal ECRIDcounter		: std_logic_vector( 7 downto 0);
		
		signal l1id_preroll		: std_logic;
      signal l1id_roll 	 		: std_logic;
	

	---------------------------------------------------------------------------
	begin

      -- EX Strobes -- used L1 Strobe from TTC as this is the last one
		-----------------------------------------------------------------------

	   TTCL1ID_strb_q(0) <=	TTCL1ID_strb_i;
	
		process (clk)
		begin
			if rising_edge(clk) then
				if (rst = '1') then 				
					TTCL1ID_strb_q(2 downto 1)	<=	"00";
		         EX_Strobe_o <= '0';
	
				else
			
					TTCL1ID_strb_q(2 downto 1)	<=	TTCL1ID_strb_q(1 downto 0);
					
					if (runmode = '1') then
						EX_Strobe_o <= TTCL1ID_strb_q(0);
												
					else
						-- *** part of delaying l1a to get correct BCID
						--EX_Strobe_o <= AllL1A_i;
						EX_Strobe_o <= AllL1A_q2;
					
					end if;
				end if;
			end if;
		end process;
	
		
	  	-- CountL1ID
		------------------------------------------------------------------------
		process (clk)
	   begin
		
			if rising_edge(clk) then
				if (rst = '1') then
					CountL1ID <= (others => '1');
				
				else
					if (runmode = '1') then
						if (TTCL1ID_strb_q(0) = '1') then
							CountL1ID <= TTCL1ID_i;
		            end if;
						
					else
						if (AllL1A_i = '1') then
							CountL1ID <= CountL1ID + '1';
							
						elsif (AllECR_i = '1') then
							CountL1ID(23 downto 0) <= (others => '1');
	
						elsif (db_wr = '1') then
							
							if (Reg10 = '1') then
								CountL1ID(15 downto 0) <= db;
							end if;
		               
							if (Reg12 = '1') then
								CountL1ID(23 downto 16) <= db(7 downto 0);
							end if;
		            
		            end if;
	 				end if;
				end if;
			end if;
	   end process;
	
   	L1ID <= CountL1ID;
   	

		-- L1ID rollover
		------------------------------------------------------------------------
		-- As we start with L1ID=FFFFFF, so we need to check that a proper 
		-- rollover occurs
	
	   process (clk)
	   begin
			if rising_edge(clk) then
				if (rst = '1') then
        				l1id_preroll <= '0';
        				l1id_roll 	 <= '0';
        		
        		else
			      if (CountL1ID = "111111111111111111111110") then
			      	l1id_preroll <= '1';
			      end if;
			      
			      if (CountL1ID = "000000000000000000000000") then
			      	l1id_roll    <= l1id_preroll;
			      	l1id_preroll <= '0';
			      end if;

			   end if;
		   end if;
		end process;




	  	-- RegisterL1ID
		------------------------------------------------------------------------
		process (clk)
		begin
			if rising_edge(clk) then
				if (rst = '1') then
					RegL1ID <= (others => '1');
					
				else
					if (runmode = '1') then 
						if	(TTCL1ID_strb_q(1) = '1') then
							RegL1ID <= TTCL1ID_i;
	
						end if;
						
					else
						if (AllL1A_q = '1') then -- Copies L1ID AFTER increment
							RegL1ID <= CountL1ID;
						
						elsif (db_wr = '1') then
							if    (Reg10 = '1') then
								RegL1ID(15 downto  0) <= db;
							
							elsif (Reg12 = '1') then
								RegL1ID(23 downto 16) <= db(7 downto 0);
					
							end if;
						end if;
					end if;
				end if;
			end if;
		end process;
		
		
		dbh_io <= (ECRID & RegL1ID(23 downto 16)) when ((db_rd and Reg10 and db_d32) = '1') else (others => 'Z');
   	db 	 <= (ECRID & RegL1ID(23 downto 16)) when ((db_rd and Reg12)            = '1') else (others => 'Z');
   	db 	 <= RegL1ID(15 downto  0) when ((db_rd and Reg10)                      = '1') else (others => 'Z');
   	
   	
   	
		
		
		-- SAECR2 output
		------------------------------------------------------------------------
	
		-- *** The problem here might be that the SAECR2 happens at the same time
		--  as the idTRIG that rolls the L1ID counter over. We need to probably
		--  use a delay of x clks
		
		SAECR2_o <= l1id_roll and enSAECR2_i;
	      
	      

	  -- ECRCounter
	  -------------------------------------------------------------------------
	
	   process (clk)
	   begin

			if rising_edge(clk) then
	      	if (rst = '1') then
	         	ECRIDcounter <= "00000000";
	         
				else
		      	      
		      	if (db_wr = '1') and (Reg10 = '1') and (runmode = '0') then
						ECRIDcounter <= "00000000";
		
		         elsif (AllECR_i = '1') or 
		         		((l1id_roll and l1idro_ecr_en_i) = '1') then
		         		
						ECRIDcounter <= ECRIDcounter + '1';
		
		         end if;
		      end if;
		   end if;
	   end process;
	   
	   ECRID <= ECRIDcounter;

   end block;



	---------------------------------------------------------------------------
	-- BCID BLOCK
	---------------------------------------------------------------------------
	bcid_blk : block
	
		signal TTCBCID_strb_q : std_logic;
      signal bcid_counter   : std_logic_vector(11 downto 0);
      signal SABCR2			 : std_logic;
      signal bcr_delay      : std_logic_vector(15 downto 0);
      signal delayed_bcr    : std_logic;
      
      signal bcid_ro			 : std_logic_vector(11 downto 0);
      signal bcid_offs		 : std_logic_vector(11 downto 0);
      
      constant SA_BCID_RO 	 : std_logic_vector(11 downto 0) := "111111111111";

	begin
		
 		bcid_ro <= "111111111111" when bcid_max_en_i = '1' else 
 		           conv_std_logic_vector(BCID_ROLLOVER, 12);
 		
 		
 		
 		--*** removed delay of 1
		bcr_delay(0) <= AllBCR_i;	
		delayed_bcr  <= bcr_delay(conv_integer(RegBCIDoffset));
	
		-- BCID Counter
		------------------------------------------------------------------------
		process (clk)
		begin
	   	if rising_edge(clk) then
				if (rst = '1') then 				
					bcid_counter <= (others => '0');
					bcr_delay(15 downto 1) <= (others => '0');
					
				else
					-- *** Removed Delay of 1
					bcr_delay(15 downto 1) <= bcr_delay(14 downto 0);
									
					if (delayed_bcr = '1') then
					   bcid_counter <= (others => '0');
					            
					elsif (bcid_counter = SA_BCID_RO) then
					   bcid_counter <= (others => '0');
					   			   
					else
					   bcid_counter <= bcid_counter + '1';

		   		end if;
			   end if;
			end if;
	   end process;
	   
	  
	  	-- Registers
		------------------------------------------------------------------------
		bcid_offs <= ("00000000" & RegBCIDoffset);
		
		process (clk)
		begin
	   	if rising_edge(clk) then
				if (rst = '1') then 				
					RegBCID     	<= (others => '0');
					RegTTCBCID  	<= (others => '0');
					RegBCIDoffset 	<= (others => '0');
					
				else
			   
			   	-- RegBCID  (read-only) (NOTE RegBCID == BCID)
					-----------------------
				   if (runmode = '1') then
				   	if (TTCL1A_i = '1') then
	  				   	if (bcid_offs > TTCBCID_i) then
					   		RegBCID <= (TTCBCID_i + (bcid_ro + 1)) - bcid_offs;
					   	
					   	else
					   		RegBCID <= TTCBCID_i - bcid_offs;
					   	
					   	end if;
					   end if;
				      			   
				   else
				   	-- *** added extra delay to test bcid=0 value when BCR + L1A coincide 
				   	--       (neglecting offset)
				   	--(and then removed it as TTCrx issues L1A on next clk after BCR for BCID=0)
				   	
				   	
				   	-- *** added 24/3/07 to get BCID lined up when using sa_ttcl1a
				   	if (sa_ttcl1a_en_i = '0') then
				   		if (AllL1A_i = '1') then
	      					RegBCID <= bcid_counter;
	      				end if;
							
						else	
						   if (AllL1A_q2 = '1') then
						   	RegBCID <= bcid_counter;	
							end if;
	            	end if;
	            end if;
	
					-- RegTTCBCID  (read-only)
					------------------------------
 	            if (TTCBCID_strb_i = '1') then
	               RegTTCBCID <= TTCBCID_i;
	               
					end if;
					
					-- RegBCIDoffset		      	
					----------------
		      	if (db_wr and Reg14) = '1' then
		         	RegBCIDoffset <= db(15 downto 12);
		         	
            	end if;

				end if;
			end if;
		end process;
				
		db <= (RegBCIDoffset & RegBCID) when ((db_rd and Reg14) = '1') else (others => 'Z');
		db <= ("0000" & RegTTCBCID) 	  when ((db_rd and Reg2A) = '1') else (others => 'Z');
		
		BCID <= RegBCID;
		
	
	 	-- SABCR2
		------------------------------------------------------------------------
		process (clk)
	   begin
						
			if rising_edge(clk) then
				if (rst = '1') then
					SABCR2 <= '0';
				
				else
				   SABCR2 <= '0';
				   				      			   
				   if (runmode = '0') then
				   	if (bcid_counter = SA_BCID_RO) then
					   	SABCR2 <= '1';
					   end if;
				   end if;

				end if;
			end if;
	   end process;
	   
	   SABCR2_o <= SABCR2 and enSABCR2_i;
	
		
	end block;
	
	


   ---------------------------------------------------------------------------
	-- TTID BLOCK
	---------------------------------------------------------------------------
	ttid_blk : block
		signal ttype_count : std_logic_vector(15 downto 0);
			
	begin

		-- Strobes (TT Local)
		------------------------------------------------------------------------

      process (clk)
		begin
		
		   if rising_edge(clk) then
		   	if (rst = '1') then 				
					TT_Strobe_o <= '0';
					
				else
					if (runmode = '1') or (sa_ttcttid_en_i = '1') then
						TT_Strobe_o <= TTCTTID_strb_i;
					
					else
						TT_Strobe_o <= AllL1A_i;
						
					end if;
				end if;
			end if;
		end process;
		
		  		     
      
      -- TTID - latch and register
      -------------------------------------------------------------------------
      process (clk)
      begin

         if rising_edge(clk) then

	         if (rst = '1') then
            	TTID <= (others => '0');
				--*** I THINK THIS LOGIC IS A LITTLE CRAPPY!
				else
	         	if (runmode = '1') or (sa_ttcttid_en_i = '1') then
	         		if (TTCTTID_strb_i = '1') then
	   					TTID <= TTCTTID_i;
	   				end if;
	   
	            else
		            if (db_wr and Reg16) = '1' then
		   				TTID <= db( 9 downto 0);
		   
		            end if;
		         end if;
	         end if;
         end if;
      end process;
      
      
      db <= ("000000" & TTID) when ((db_rd and Reg16) = '1') else (others => 'Z');
      
      
      
	---------------------------------------------------------------------------   
	end block;
   
   
 
end rtl;
