
------------------------------------------------------------------------------
-- ARCHITECTURE
------------------------------------------------------------------------------
architecture timold of exttid is

------------------------------------------------------------------------------
-- GLOBAL SIGNAL DECLARATIONS
------------------------------------------------------------------------------
		signal AllL1A_q      : std_logic_vector(2 downto 0);
		
		signal L1ID          : std_logic_vector(23 downto 0);
		signal BCID          : std_logic_vector(11 downto 0);
		signal TTID          : std_logic_vector( 9 downto 0);
	   signal ECRID         : std_logic_vector( 7 downto 0);
	   
	   --signal L1_strobe     : std_logic;
	   --signal BC_strobe     : std_logic;
  	   --signal TT_strobe     : std_logic; 
  	   
  		signal RegL1ID 	   : std_logic_vector(23 downto 0);
  		signal RegBCID 	   : std_logic_vector(11 downto 0);
  		signal RegTTCBCID    : std_logic_vector(11 downto 0);
  		signal RegTTID       : std_logic_vector( 9 downto 0);
      signal RegBCIDoffset : std_logic_vector( 3 downto 0);


------------------------------------------------------------------------------
-- MAIN CODE
------------------------------------------------------------------------------
begin

	------------------------------------------------------------------------
  	-- Global
	------------------------------------------------------------------------
	
	
	AllL1A_q(0) <= AllL1A_i;


	process (rst, clk)
	begin
		if (rst = '1') then 
			AllL1A_q(2 downto 1) <= "00";
	   
	   elsif rising_edge(clk) then 
	   	AllL1A_q(2 downto 1) <= AllL1A_q(1 downto 0);

		end if;
	end process;


	-- Outputs (to FIFO)
	------------------------------------------------------------------------
	process (rst, clk)
	begin
	   if (rst = '1') then 				
			
		   EVID_o <= (others => '0');
         TTID_o <= (others => '0');
		   
		 elsif rising_edge(clk) then
		   	
		   EVID_o(43 downto 36) <= ECRID;
		   EVID_o(35 downto 12) <= L1ID;
		   EVID_o(11 downto  0) <= BCID;
		   		   
		   TTID_o <= TTID;
		   		   	
		end if;
   end process;
	
	
  

	
	---------------------------------------------------------------------------
	-- L1ID BLOCK
	---------------------------------------------------------------------------
	l1id_blk : block
	
		-- Strobes
		signal TTCL1ID_strb_q	: std_logic_vector( 2 downto 0);
		
		-- L1ID register and counter
		signal CountL1ID 			: std_logic_vector(23 downto 0);
	
		signal ECRIDcounter		: std_logic_vector( 7 downto 0);
	

	---------------------------------------------------------------------------
	begin

      -- Ev Strobes -- used L1 Strobe from TTC as this is the last one
		-----------------------------------------------------------------------

	   TTCL1ID_strb_q(0) <=	TTCL1ID_strb_i;
	
		process (rst, clk)
		begin
			if (rst = '1') then 				
				TTCL1ID_strb_q(2 downto 1)	<=	"00";
	
	         Ev_Strobe_o <= '0';
		
		   elsif rising_edge(clk) then
				TTCL1ID_strb_q(2 downto 1)	<=	TTCL1ID_strb_q(1 downto 0);
					
				if (RunMode = '1') then
					Ev_Strobe_o <= TTCL1ID_strb_q(1);
					
				else
					Ev_Strobe_o <= AllL1A_q(2);
						
				end if;
			   	
			end if;
		end process;
	
		
	  	-- CountL1ID
		------------------------------------------------------------------------
		process (rst, clk)
	   begin
		
			if (rst = '1') then
				CountL1ID <= (others => '0');
	         					
			elsif rising_edge(clk) then
				if (RunMode = '1') then
					if (TTCL1ID_strb_q(0) = '1') then
						CountL1ID <= TTCL1ID_i;
	            end if;
					
				else
					if (AllL1A_q(1) = '1') then
						CountL1ID <= CountL1ID + '1';

					elsif (vmeWR = '1') then
						if    (Reg10 = '1') then
							CountL1ID(15 downto 0) <= db;
	               
						elsif (Reg12 = '1') then
							CountL1ID(23 downto 16) <= db(7 downto 0);

						end if;
	            
	            end if;
 				end if;
			end if;
	   end process;
	
   	L1ID <= CountL1ID;


	  	-- RegisterL1ID
		------------------------------------------------------------------------
		process (rst, clk)
		begin
		
			if (rst = '1') then
				RegL1ID <= (others => '1');
				
			elsif rising_edge(clk) then
				if (RunMode = '1') then 
					if	(TTCL1ID_strb_q(1) = '1') then
						RegL1ID <= TTCL1ID_i;

					end if;
					
				else
					if (AllL1A_q(2) = '1') then
						RegL1ID <= CountL1ID;
					
					elsif (vmeWR = '1') then
						if    (Reg10 = '1') then
							RegL1ID(15 downto  0) <= db;
						
						elsif (Reg12 = '1') then
							RegL1ID(23 downto 16) <= db(7 downto 0);
				
						end if;
					end if;
				end if;
			end if;
		end process;
					
		

		-- SAECR2 output
		------------------------------------------------------------------------
	
		-- *** The problem here might be that the SAECR2 happens at the same time
		--  as the idTRIG that rolls the L1ID counter over. We need to probably
		--  use a delay of x clks
	
	   process (rst, clk)
	   begin
	
	      if (rst = '1') then
	         SAECR2_o <= '0';
	
	         
	      elsif rising_edge(clk) then
	      
		      if (CountL1ID = "111111111111111111111111") then
		      
		      	if (AllL1A_q(0) = '1') and (enSAECR2_i = '1') then
		      		SAECR2_o <= '1';
		
		         end if;
		      end if;
		   end if;
		end process;
	      
	      

	  -- ECRCounter
	  -------------------------------------------------------------------------
	
	   process (rst, clk)
	   begin
	
	      if (rst = '1') then
	         ECRIDcounter <= "00000000";
	         
	      elsif rising_edge(clk) then
	      
	      	if (vmeWR = '1') and (Reg10 = '1') and (RunMode = '0') then
					ECRIDcounter <= "00000000";
	
	         elsif (AllECR_i = '1') then
					ECRIDcounter <= ECRIDcounter + '1';
	
	         end if;
	      end if;
	   end process;
	   
	   ECRID <= ECRIDcounter;

   end block;



	---------------------------------------------------------------------------
	-- BCID BLOCK
	---------------------------------------------------------------------------
	bcid_blk : block
	
		-- Strobes
		signal TTCBCID_strb_q	: std_logic_vector(2 downto 0);
				
      signal BCIDcounter      : std_logic_vector(11 downto 0);
      
      signal SABCR2				: std_logic;
	

	

	---------------------------------------------------------------------------
	begin
	
	   -- BC Strobe (local) -- not used for Ev Strobe gen
		-----------------------------------------------------------------------

	   TTCBCID_strb_q(0) <=	TTCBCID_strb_i;
	
		process (rst, clk)
		begin
			if (rst = '1') then 				
				TTCBCID_strb_q(2 downto 1)	<=	"00";
	
	      elsif rising_edge(clk) then
				TTCBCID_strb_q(2 downto 1)	<=	TTCBCID_strb_q(1 downto 0);
						   	
			end if;
		end process;
		
		
	  	-- BCIDcounter
		------------------------------------------------------------------------
		process (rst, clk)
	   begin
		
			if (rst = '1') then
				BCIDcounter <= (others => '0');
				
			elsif rising_edge(clk) then
			
			   if Runmode = '1' then
			   
			      BCIDcounter <= RegTTCBCID + RegBCIDoffset;
			      			   
			   else
		         					
				   if (AllBCR_i = '1') then
					   BCIDcounter <= (others => '0');
					            
				   elsif (BCIDcounter = "111111111111") then
				      BCIDcounter <= (others => '0');
				      			   
				   else
				      BCIDcounter <= BCIDcounter + '1';
            					       
			      end if;
			   end if;
			end if;
	   end process;
	   
	  
	  	-- RegBCID, RegTTCBCID  (read-only)
		------------------------------------------------------------------------
		process (rst, clk)
		begin
		
			if (rst = '1') then
				RegBCID     <= (others => '0');
				RegTTCBCID  <= (others => '0');
			
			elsif rising_edge(clk) then
			   
		      if (AllL1A_q(1) = '1') then
					RegBCID <= BCIDcounter;
            end if;


            if (TTCBCID_strb_q(0) = '1') then
               RegTTCBCID <= TTCBCID_i;
				end if;

			end if;
		end process;
		
		BCID <= RegBCID;
		
		
		-- RegBCIDoffset
		------------------------------------------------------------------------
		process (rst, clk)
		begin
		
			if (rst = '1') then
				RegBCIDoffset <= (others => '0');
			
			elsif rising_edge(clk) then
			   
		      if (vmeWR and Reg14) = '1' then
		         RegBCIDoffset <= db(15 downto 12);
            
            end if;
			end if;
		end process;
		
		
	 	-- SABCR2
		------------------------------------------------------------------------
		process (rst, clk)
	   begin
		
			if (rst = '1') then
				SABCR2 <= '0';
				
			elsif rising_edge(clk) then
			
			   if Runmode = '1' then
			      SABCR2 <= '0';
			      			   
			   elsif (BCIDcounter = "111111111111") then
				   SABCR2 <= '1';
				   
				else
				   SABCR2 <= '0';
				
			   end if;
			end if;
	   end process;
	   
	   SABCR2_o <= SABCR2 and enSABCR2_i;
	
		
	end block;

	


   ---------------------------------------------------------------------------
	-- TTID BLOCK
	---------------------------------------------------------------------------
	ttid_blk : block
	
		-- Strobe
		signal TTCTTID_strb_q   : std_logic_vector(2 downto 0);
		
		
	---------------------------------------------------------------------------
	begin
			

		-- Strobes (TT Local)
		------------------------------------------------------------------------

      TTCTTID_strb_q(0) <=	TTCTTID_strb_i;      

		process (rst, clk)
		begin
		
			if (rst = '1') then 				
				TTCTTID_strb_q(2 downto 1)	<=	"00";

            TT_Strobe_o <= '0';
	
		   elsif rising_edge(clk) then
				TTCTTID_strb_q(2 downto 1)	<=	TTCTTID_strb_q(1 downto 0);
				
				if (RunMode = '1') then
					TT_Strobe_o <= TTCTTID_strb_q(1);
				
				else
					TT_Strobe_o <= AllL1A_q(2);
					
				end if;
		   	
			end if;
		end process;
  		     
      
      -- TTID - incl. register
      -------------------------------------------------------------------------
      process (rst, clk)
      begin
   
         if (rst = '1') then
            TTID <= (others => '0');
            
         elsif rising_edge(clk) then
         
         	if (RunMode = '1') then
         		if (TTCTTID_strb_q(0) = '1') then
   					TTID <= TTCTTID_i;
   				end if;
   
            elsif (vmeWR and Reg16) = '1' then
            
   				TTID <= db( 9 downto 0);
   
            end if;
         end if;
      end process;
      
      RegTTID <= TTID;

   end block;
   ---------------------------------------------------------------------------   
   
   
					


   -- Data Read
   -----------------------------------------------------------------------------

   read_db : block
		signal Reg10_db	: std_logic_vector(15 downto 0);
   	signal Reg12_db	: std_logic_vector(15 downto 0);
   	signal Reg14_db	: std_logic_vector(15 downto 0);
   	signal Reg16_db	: std_logic_vector(15 downto 0);
   	signal Reg2A_db	: std_logic_vector(15 downto 0);

   begin
   	Reg10_db					 <= RegL1ID(15 downto 0);
   
   	Reg12_db(15 downto 8) <= ECRID;
   	Reg12_db( 7 downto 0) <= RegL1ID(23 downto 16);
   	
   	Reg14_db(15 downto 12) <= RegBCIDoffset; 
   	Reg14_db(11 downto  0) <= RegBCID;          	

   	Reg16_db(15 downto 10) <= "000000";    
   	Reg16_db( 9 downto 0)  <= RegTTID;     
   		
		Reg2A_db(15 downto 12) <= "0000";    
		Reg2A_db(11 downto  0) <= RegTTCBCID;
		
	   	
   	db <= Reg10_db when ((vmeRD and Reg10) = '1') else
   			Reg12_db when ((vmeRD and Reg12) = '1') else
		   	Reg14_db when ((vmeRD and Reg14) = '1') else
   			Reg16_db when ((vmeRD and Reg16) = '1') else
   			Reg2A_db when ((vmeRD and Reg2A) = '1') else (others => 'Z');

	end block;

   
   -- Debug Bus
  	----------------------------------------------------

	DebugBus <= "0000000000000000";

end timold;
