
-------------------------------------------------------------------------------
--	ARCHITECTURE
-------------------------------------------------------------------------------
architecture timold of mixer is

-------------------------------------------------------------------------------
--	SIGNAL DECLARATIONS
-------------------------------------------------------------------------------

	signal L1A					: std_logic;
	signal BCR					: std_logic;
	signal ECR					: std_logic;
	signal CAL					: std_logic;
	signal FER					: std_logic;
	signal SPR					: std_logic;
	signal TTS					: std_logic;
	signal EXS					: std_logic;
	signal enTTCSPR			: std_logic;
	signal enTTCFER			: std_logic;
	signal enTTCCAL			: std_logic;
	signal enTTCBCR			: std_logic;
	signal enTTCECR			: std_logic;
	signal enTTCL1A			: std_logic;
	signal enSerEXS			: std_logic;	
	signal enSerTTS			: std_logic;		
	signal RegEnTTC			: std_logic_vector(15 downto 0);
	signal RegTestTTCbus		: std_logic_vector(15 downto 0);

-------------------------------------------------------------------------------
--	MAIN CODE
-------------------------------------------------------------------------------
begin


	L1A <= SATRIG_i or				 (TTCL1A_i and	enTTCL1A) or SeqTRIG_i;
	BCR <= SABCR_i	 or SABCR2_i or (TTCBCR_i and	enTTCBCR) or SeqBCR_i;
	ECR <= SAECR_i	 or SAECR2_i or (TTCECR_i and	enTTCECR) or SeqECR_i;
	CAL <= SACAL_i	 or				 (TTCCAL_i and	enTTCCAL) or SeqCAL_i;
	FER <= SAFER_i	 or				 (TTCFER_i and	enTTCFER) or SeqFER_i;
	SPR <= SASPR_i	 or				 (TTCSPR_i and	enTTCSPR) or SeqSPR_i;
	
	EXS <=                         (SerEXS_i and enSerEXS) or SeqEXS_i; -- *** this may no be the understood 
	TTS <=                         (SerTTS_i and enSerTTS) or SeqTTS_i; --     effect of these enables



	--	Unclocked for backplane	(done	externally on the	NOT clk)
	TTCbus_o(0)	<=	L1A;
	TTCbus_o(1)	<=	ECR;
	TTCbus_o(2)	<=	BCR;
	TTCbus_o(3)	<=	CAL;
	TTCbus_o(4)	<=	EXS;
	TTCbus_o(5)	<=	TTS;
	TTCbus_o(6)	<=	FER;
	TTCbus_o(7)	<=	SPR;


	--	Clocked for	on	board	use
	process (rst, clk) --,	L1A, BCR, ECR,	CAL, FER, SPR,	EXS, TTS)
	begin

		--	Clocked signals
		if	(rst = '1') then

			AllL1A_o	 <= '0';
			AllBCR_o	 <= '0';
			AllECR_o	 <= '0';
			AllCAL_o	 <= '0';
			AllFER_o	 <= '0';
			AllSPR_o	 <= '0';
			AllEXS_o	 <= '0';
			AllTTS_o	 <= '0';

		elsif	rising_edge(clk) then

			AllL1A_o	<=	L1A;
			AllBCR_o	<=	BCR;
			AllECR_o	<=	ECR;
			AllCAL_o	<=	CAL;
			AllFER_o	<=	FER;
			AllSPR_o	<=	SPR;
			AllEXS_o	<=	EXS;
			AllTTS_o	<=	TTS;

		end if;

	end process;



	--	RegEnTTC
	----------------------------------------------------

	process (rst, clk) --, Reg18, db_wr)
	begin

		if (rst = '1') then						--	async	reset
			RegEnTTC <= "0000000000000000";

		elsif rising_edge(clk) then
		
			if	(Reg18 and db_wr)	= '1'	then
				RegEnTTC <= db;

			end if;

		end if;

	end process;

	enSABCR2_o		   <=	RegEnTTC(15);
	enSAECR2_o		   <=	RegEnTTC(14);
	enSerTTS			   <=	RegEnTTC(10);
	enSerEXS			   <=	RegEnTTC( 9);
	ext_RodBUSYinen_o	<=	RegEnTTC( 8);
	RCBUSYen_o			<=	RegEnTTC( 7);
	enTTCSPR			   <=	RegEnTTC( 6);
	enTTCFER			   <=	RegEnTTC( 5);
	enTTCCAL			   <=	RegEnTTC( 4);
	enTTCBCR			   <=	RegEnTTC( 3);
	enTTCECR			   <=	RegEnTTC( 2);
	enTTCL1A			   <=	RegEnTTC( 1);
	setTTCClk_o		   <=	RegEnTTC( 0);

	--	Reg_TestTTCbus	(was TTCbus) -- self-holding
	----------------------------------------------------
	process (clk)
	begin

		if rising_edge(clk) then
		
			if	(rst = '1') then
				RegTestTTCbus	<=	(others => '0');

			else
				if (Reg30 and db_wr) = '1' then
					RegTestTTCbus <= (others => '0');

				else	--	Self Holding Test Register
					RegTestTTCbus(0) <= L1A or RegTestTTCbus(0);
					RegTestTTCbus(1) <= ECR or RegTestTTCbus(1);
					RegTestTTCbus(2) <= BCR or RegTestTTCbus(2);
					RegTestTTCbus(3) <= CAL or RegTestTTCbus(3);
					RegTestTTCbus(4) <= EXS or RegTestTTCbus(4);
					RegTestTTCbus(5) <= TTS or RegTestTTCbus(5);
					RegTestTTCbus(6) <= FER or RegTestTTCbus(6);
					RegTestTTCbus(7) <= SPR or RegTestTTCbus(7);
		
	      	end if;
	      end if;
	   end if;
	end process;


	--	Data Bus
	----------------------------------------------------

	db <= RegEnTTC 	  when ((Reg18 and db_rd) = '1') else 
		   RegTestTTCbus when ((Reg30 and db_rd) = '1') else (others => 'Z');

	
   
   -- Debug Bus
  	----------------------------------------------------

	DebugBus <= "0000000000000000";

end timold;

-------------------------------------------------------------------------------
