
-------------------------------------------------------------------------------

architecture rtlold of rod_busys is

-------------------------------------------------------------------------------
-- SIGNAL DECLARATIONS
-------------------------------------------------------------------------------

	signal RegRODBusy		: std_logic_vector(15 downto 0);
	signal RegRB_Mask	   : std_logic_vector(15 downto 0);
	signal RegRB_Latch   : std_logic_vector(15 downto 0);
	signal RegRB_Monitor	: std_logic_vector(15 downto 0);

	signal RB_Store      : std_logic_vector(15 downto 0);
	signal BO            : std_logic_vector(15 downto 0);
	signal clk_div16		: std_logic;
	
-------------------------------------------------------------------------------
-- MAIN CODE
-------------------------------------------------------------------------------
begin


	-----------------------------------------------------------------------------
	-- RODBusy and RB_Mask Registers
	-----------------------------------------------------------------------------


	-- RegRODBusy - Reg20
	------------------
	
	process (rst, clk)
	begin
		if (rst = '1') then
			RegRODBusy <= (others => '0');

		elsif rising_edge(clk) then
			RegRODBusy <= bp_RODBusy_i;

		end if;

	end process;



	-- RegRB_Mask
	----------------------
	process (rst, clk)
	begin

		if (rst = '1') then
			RegRB_Mask <= (others => '0');

		elsif rising_edge(clk) then
         if (Reg1E and vmeWR) = '1' then
				RegRB_Mask <= db;

			end if;
		end if;
	end process;






	-----------------------------------------------------------------------------
	-- RegRB_Latch - one way latch - stores busys - reset by write
	-----------------------------------------------------------------------------

	process (rst, clk) --Reg22, vmeWR, RegRODBusy, RegRB_Latch)
	begin
		if (rst = '1') then
			RegRB_Latch <= "0000000000000000";

		elsif rising_edge(clk) then
		
			if (Reg22 and vmeWR) = '1' then
				RegRB_Latch <= "0000000000000000";

			else      -- Self Holding Register
				RegRB_Latch <= RegRODBusy or RegRB_Latch;

			end if;
		end if;

	end process;


	
   -----------------------------------------------------------------------------
	-- RB_Monitor latches busy high for 16 to 31 clocks -- reset by write
	-----------------------------------------------------------------------------

	counter : process (rst, clk)
      variable count : integer;

	begin
		if (rst = '1') then
			count:= 0;
			clk_div16 <= '0';
			
		elsif rising_edge(clk) then
			count:= (count + 1) mod 16;
			
			if count = 15 then
				clk_div16 <= '1';
			
			else
         	clk_div16 <= '0';

			end if;
		end if;
		
	end process counter;



	process (rst, clk)
	begin
		if (rst = '1') then			
			RB_Store <= (others => '0');

		elsif rising_edge(clk) then
			if clk_div16 = '1' then
				RB_Store <= RegRODBusy;
			
			end if;

		end if;
	end process;



	process (rst, clk) --Reg24, vmeWR, RegRODBusy, RB_Store, RegRB_Monitor)
	begin

		if (rst = '1') then
			RegRB_Monitor <= (others => '0');

		elsif rising_edge(clk) then
		
			if (Reg24 and vmeWR) = '1' then
				RegRB_Monitor <= (others => '0');

			else -- self holding
				RegRB_Monitor <= RB_Store or RegRB_Monitor;

			end if;
		end if;
	end process;



	
   ----------------------------------------------------------------------------
   -- RB_Monitor - Latches busy high for 16 to 31 clocks	-- reset by write     
   ----------------------------------------------------------------------------

   BO <= RegRODBusy AND RegRB_Mask; -- BO for legi only
           
   RCBUSY_o <=   BO(15) OR BO(14) OR BO(13) OR BO(12) OR 
                 BO(11) OR BO(10) OR BO( 9) OR BO( 8) OR 
                 BO( 7) OR BO( 6) OR BO( 5) OR BO( 4) OR
                 BO( 3) OR BO( 2) OR BO( 1) OR BO( 0) OR 
                 v_RCBUSY_i OR 
                 (ext_RodBUSYin_i AND ext_RodBUSYinen_i) OR
                 (not RunMode);	 


  -----------------------------------------------------------------------------
  -- Data Bus
  -----------------------------------------------------------------------------

	db <= RegRB_Mask	 	when ((Reg1E and vmeRD) = '1') else
			RegRODBusy  	when ((Reg20 and vmeRD) = '1') else
			RegRB_Latch 	when ((Reg22 and vmeRD) = '1') else
			RegRB_Monitor 	when ((Reg24 and vmeRD) = '1') else (others => 'Z');
		 
  
   
   -- Debug Bus
  	----------------------------------------------------

	DebugBus <= (others => '0');

end rtlold;
