-- hds header_start
--
-- VHDL Architecture TIM_fpga2.sequencer_tester.test
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 15:36:13 03/06/04
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity sequencer_tester is
   port( 
      RAMselect       : out    std_logic;
      ren_src_addr    : out    std_logic;
      ren_sink_addr   : out    std_logic;
      StartSeq        : out    std_logic;
      StartSink       : out    std_logic;
      ab              : out    std_logic_vector (15 downto 0);
      clk             : out    std_logic;
      rst             : out    std_logic;
      db_rd           : out    std_logic;
      db_wr           : out    std_logic;
      reg_seq_ctl     : out    std_logic_vector (15 downto 0);
      reg_seq_end     : out    std_logic_vector (15 downto 0);
      reg_run_enables : out    std_logic_vector (15 downto 0);
      db              : inout  std_logic_vector (15 downto 0);
      busy_bst        : out    std_logic
   );

-- Declarations

end sequencer_tester ;


-- hds interface_end


architecture test of sequencer_tester is

	signal clkosc 	 		: std_logic := '1';   

	constant DB_RAMsel 	: std_logic_vector(2 downto 0) := "001";
	constant DB_REG_SRC 	: std_logic_vector(2 downto 0) := "010";
	constant DB_REG_SINK	: std_logic_vector(2 downto 0) := "100";

	
begin
-------------------------------------------------------
   
	-- clock generator
	------------------
	clkosc	<= not clkosc after 12500 ps;
	clk 		<= clkosc;

	--------------------------------------------------------
	sim : process
	
		-- Procedures
		----------------------------------------------------------------
		procedure DB_Write(
				regram_en	: in std_logic_vector(2 downto 0);  
				iaddr 		: in integer;
				idata 		: in integer
			) is
			variable addr : std_logic_vector(15 downto 0);
			variable data : std_logic_vector(15 downto 0);
		begin
			addr := conv_std_logic_vector(iaddr, 16);
			data := conv_std_logic_vector(idata, 16);
			ab       	<= addr;
      	RAMselect 	<= regram_en(0);
      	ren_src_addr 	<= regram_en(1);
      	ren_sink_addr 	<= regram_en(2);	wait for 25 ns;
			db     		<= data;				wait for 25 ns;
			db_wr	  		<= '1';				wait for 25 ns;
			db_wr	  		<= '0';				wait for 25 ns;
      	RAMselect 	<= '0';
      	ren_src_addr 	<= '0';
      	ren_sink_addr 	<= '0';
			ab  			<= (others => 'Z');
			db    		<= (others => 'Z'); wait for 25 ns;
		end procedure; 
		----------------------------------------------------------------
		
		procedure DB_Read(
				regram_en	: in std_logic_vector(2 downto 0);  
				iaddr 		: in integer
			) is
			variable addr : std_logic_vector(15 downto 0);
		begin
			addr := conv_std_logic_vector(iaddr, 16);
			ab       		<= addr;
      	RAMselect 		<= regram_en(0);
      	ren_src_addr 	<= regram_en(1);
      	ren_sink_addr 	<= regram_en(2);	wait for 75 ns;
			db_rd	  			<= '1';				wait for 25 ns;
			db_rd	  			<= '0';				wait for 25 ns;
      	RAMselect 		<= '0';
      	ren_src_addr 	<= '0';
      	ren_sink_addr 	<= '0';
			ab  			<= (others => 'Z'); wait for 25 ns;
		end procedure; 

	----------------------------------------------------------------------
	-- Simulation
	----------------------------------------------------------------------	
	begin	
	
		-- Start-up values
		----------------------------------------------------------

		rst      	<= '0';    
      db_rd    	<= '0';
      db_wr    	<= '0';      
      ab       	<= (others => '0');
      db       	<= (others => 'Z');
      RAMselect 	<= '0';
      ren_src_addr 	<= '0';
      ren_sink_addr 	<= '0';
      
      reg_run_enables <= (others => '0');
      reg_seq_ctl <=  (others => '0');
      reg_seq_end <=  (others => '0');
      
      StartSeq    <= '0';
      StartSink   <= '0';
      
      busy_bst    <= '0';
      
      

      -----------------------------------------------------
      -- Main Simulation
		-----------------------------------------------------
	   
		-- Reset
		wait for 25 ns;		
	   rst <= '1';
	   wait for 105 ns;  -- add some delay so sigs don't arrive exactly on the clock!
	   rst <= '0';
  
	   
	   -- Data IO
	   ----------
	   
	   --DB_Write(DB_RAMsel, 16#8000#, 16#0001#);
  	   --DB_Write(DB_RAMsel, 16#8002#, 16#0023#);
  	   --DB_Write(DB_RAMsel, 16#8004#, 16#0045#);
  	   --DB_Write(DB_RAMsel, 16#8006#, 16#0067#);
  	   --DB_Write(DB_RAMsel, 16#8008#, 16#0089#);
  	   --DB_Write(DB_RAMsel, 16#800A#, 16#00AB#);
  	   --DB_Write(DB_RAMsel, 16#800C#, 16#00CD#);
  	   --DB_Write(DB_RAMsel, 16#800E#, 16#00EF#);
  	   
  	   DB_Write(DB_RAMsel, 16#8000#, 9);
  	   DB_Write(DB_RAMsel, 16#8002#, 8);
  	   DB_Write(DB_RAMsel, 16#8004#, 7);
  	   DB_Write(DB_RAMsel, 16#8006#, 6);
  	   DB_Write(DB_RAMsel, 16#8008#, 5);
  	   DB_Write(DB_RAMsel, 16#800A#, 4);
  	   DB_Write(DB_RAMsel, 16#800C#, 3);
  	   DB_Write(DB_RAMsel, 16#800E#, 2);
  	   DB_Write(DB_RAMsel, 16#8010#, 1);
  	   DB_Write(DB_RAMsel, 16#8012#, 0);
  	   DB_Write(DB_RAMsel, 16#8014#, 10);
  	   DB_Write(DB_RAMsel, 16#8016#, 11);
  	   DB_Write(DB_RAMsel, 16#8018#, 12);
  	   DB_Write(DB_RAMsel, 16#801A#, 13);
  	   DB_Write(DB_RAMsel, 16#801C#, 14);
  	   DB_Write(DB_RAMsel, 16#801E#, 15);
  	   
  	   
  	   
  	   
  	   wait for 500 ns;
  	   
  	   reg_seq_end <= conv_std_logic_vector(9, 16);
  	   reg_seq_ctl <= "0010001011111111"; -- src/sink reset
  	   reg_seq_ctl <= "0100000011111111"; -- seq go
  	   
  	   wait for 200 ns;
  	   
  	   reg_seq_ctl <= "0010001011111111"; -- seq reset
  	   reg_seq_ctl <= "0100110011111111"; -- seq go, encyclic
  	   
 	   
  	   wait for 5000 ns;
  	   
  	   busy_bst <= '1';
  	   
  	   wait for 400 ns;
  	     	   
  	   busy_bst <= '0';
  	   
  	   
  	   wait for 5000 ns;
  	   
  	   
  	   reg_seq_ctl <= "0010001011111111"; -- seq reset
  	   
  	   busy_bst <= '1';
  	   
  	   wait for 100 ns;
  	   
  	   
  	   reg_seq_ctl <= "0100010011111111"; -- seq go
  	   
  	   
  	   busy_bst <= '0';
  	   
  	   wait for 5000 ns;
  	   
  	   
  	   reg_seq_ctl <= "0010001011111111"; -- seq reset
  	   
  	   for n in 0 to 50 loop
	   	DB_Read(DB_RAMsel, 16#8000#+(n*2) );
	   	wait for 100 ns;
	   end loop;

  	   
 	   
 	   wait;
   
	end process;

end test;
