
-------------------------------------------------------------------------------

architecture tim of ttcrq_sim is

-------------------------------------------------------------------------------
-- SIGNAL DECLARATIONS
-------------------------------------------------------------------------------

	signal RegTTCdata		: std_logic_vector(15 downto 0);
  	signal RegTTCselect	: std_logic_vector( 3 downto 0);
   signal RegI2C			: std_logic_vector(15 downto 0);
   signal RegTTCCmd		: std_logic_vector(15 downto 0);

	--signal SCLoe		 	: std_logic;
 	--signal SDAoe		 	: std_logic;

 	--signal TTCrxClk		: std_logic;



	-- Clocked (by TTCrxClk) versions of TTCrx signal inputs
	--------------------------------------------------------
 	signal SinErrStr_q		: std_logic;
 	signal DbErrStr_q		: std_logic;
  	signal DoutStr_q		: std_logic;
  	signal BrcstStr1_q		: std_logic;
 	signal BrcstStr2_q		: std_logic;
 	signal BCntStr_q		: std_logic;
 	signal EvCntLStr_q		: std_logic;
 	signal EvCntHStr_q		: std_logic;
 	signal BCnt_q			: std_logic_vector(11 downto 0);
 	signal Brcst_q			: std_logic_vector( 7 downto 2);
 	signal Dout_q			: std_logic_vector( 7 downto 0);
 	signal SubAddr_q		: std_logic_vector( 7 downto 0);
 	signal DQ_q      		: std_logic_vector( 3 downto 0);
 	signal L1Accept_q		: std_logic;
 	signal BCntRes_q		: std_logic;
 	signal EvCntRes_q		: std_logic;




-------------------------------------------------------------------------------
-- MAIN CODE
-------------------------------------------------------------------------------
begin

	process (TTCCLK_i)
	begin
		if rising_edge(TTCCLK_i) then -- edge depends on delays in buffers on board
			if (rst = '1') or (trx_TTCReady_i = '0') then
				SinErrStr_q	<= '0';
	         DbErrStr_q	<= '0';
	         DoutStr_q	<= '0';
	         BrcstStr1_q	<= '0';
	         BrcstStr2_q	<= '0';
	         BCntStr_q	<= '0';
	         EvCntLStr_q	<= '0';
	         EvCntHStr_q	<= '0';
	         BCnt_q		<= "000000000000";
	         Brcst_q		<= "000000";
	         Dout_q		<= "00000000";
	         SubAddr_q	<= "00000000";
	         DQ_q      	<= "0000";
	         L1Accept_q	<= '0';
	         BCntRes_q	<= '0';
	         EvCntRes_q	<= '0';
	         
	      else
	         SinErrStr_q <= trx_SinErrStr_i;
	         DbErrStr_q 	<= trx_DbErrStr_i;
	         DoutStr_q  	<= trx_DoutStr_i;
	         BrcstStr1_q	<= trx_BrcstStr1_i;
	         BrcstStr2_q	<= trx_BrcstStr2_i;
	         BCntStr_q  	<= trx_BCntStr_i;
	         EvCntLStr_q	<= trx_EvCntLStr_i;
	         EvCntHStr_q	<= trx_EvCntHStr_i;
				BCnt_q		<= trx_BCnt_i;
				Brcst_q		<= trx_Brcst_i;
				Dout_q		<= trx_Dout_i;
				SubAddr_q	<= trx_SubAddr_i;
				DQ_q			<= trx_DQ_i;
				L1Accept_q	<= trx_L1Accept_i;
	         BCntRes_q	<= trx_BCntRes_i;
	         EvCntRes_q	<= trx_EvCntRes_i;
	
	       end if;
	    end if;
    end process;



	---------------------------------------------------------------------------
	-- TTCrx reset pulse generator (~50us)
	---------------------------------------------------------------------------

	process (clk)
      variable resetb_timer : integer;
	begin
		if rising_edge(clk) then
			if (rst = '1') then
				resetb_timer:= 0;
				trx_resetb_o <= '1';
			
			else
				if (resetb_timer = 2#11111111111#) then
					resetb_timer := 2#11111111111#;
					trx_resetb_o <= '0';
	
				else
					resetb_timer := resetb_timer + 1;
					trx_resetb_o <= '1';
	
				end if;
			end if;
		end if;
	end process;

	-------------
	-- OUTPUTS
	-------------


	trxReady_o <= trx_TTCready_i;


	---------------------------------------------------------------------------
	-- TTC Signals: (Decode and) TIM-Clock Out
	---------------------------------------------------------------------------

	process (clk)
	begin
		if rising_edge(clk) then
			if (rst = '1') then
	
				TTCL1A_o <= '0';
				TTCBCR_o <= '0';
				TTCECR_o <= '0';
				TTCCAL_o <= '0';
				TTCFER_o <= '0';
			
			else
				TTCL1A_o <= L1Accept_q;
				TTCBCR_o <= BCntRes_q;
				TTCECR_o <= EvCntRes_q;
				
				-- defaults
				TTCCAL_o <= '0';
				TTCFER_o <= '0';
	
				if (BrcstStr2_q = '1') and (Brcst_q(6) = '1') then
					TTCCAL_o <= '1';
				end if;
	
				if (BrcstStr1_q = '1') and (Brcst_q(2) = '1') then
					TTCFER_o <= '1';
				end if;
	
			end if;
		end if;
   end process;

   TTCSPR_o <= '0';


	---------------------------------------------------------------------------
  	-- Event ID demux and output
 	---------------------------------------------------------------------------

	process (clk)
	begin

		if rising_edge(clk) then
			if (rst ='1') then
				TTCL1ID_o      <= (others => '0');
	         TTCL1ID_strb_o <= '0';
	
	   		TTCBCID_o      <= (others => '0');
	         TTCBCID_strb_o <= '0';
	      
	      else
		
				-- BCID -------------------------------
				if (BCntStr_q = '1') then
	   			TTCBCID_o(11 downto 0) <= BCnt_q;
				end if;
	
				TTCBCID_strb_o <= BCntStr_q;
	
	
				-- L1ID -------------------------------
				if (EvCntLStr_q = '1') then
	   			TTCL1ID_o(11 downto 0) <= BCnt_q;
				end if;
	
				if (EvCntHStr_q = '1') then
		         TTCL1ID_o(23 downto 12) <= BCnt_q;
	   		end if;
	
	   		TTCL1ID_strb_o <= EvCntHStr_q;

			end if;
		end if;
	end process;





	---------------------------------------------------------------------------
  	-- Trigger Type decode and clock out
 	---------------------------------------------------------------------------

	TTCTTID_o(9 downto 8) <= "00";
	
	process (clk)
	begin
		if rising_edge(clk) then
			if (rst ='1') then
				TTCTTID_o(7 downto 0) <= (others => '0');
				TTCTTID_strb_o <= '0';
	
			else
			
				-- default
				TTCTTID_strb_o 			<= '0';
	
				--if (DoutStr_q='1') and (SubAddr_q="00000000") and (DQ_q="0000") then
				if (DoutStr_q = '1') and ((SubAddr_q & DQ_q) = "000000000000") then
					TTCTTID_o(7 downto 0) 	<= Dout_q;
					TTCTTID_strb_o 			<= '1';
	
				end if;
			end if;
		end if;
	end process;



  	-- Register TTC Data - Reg26
  	----------------------------

	process (clk)
	begin

		if rising_edge(clk) then
			if (rst ='1') then
				RegTTCdata <= (others => '0');

			else
				if (Reg26 and vmeWR) = '1'  then
					RegTTCdata <= (others => '0');

	      	elsif (DoutStr_q = '1') and (DQ_q = RegTTCselect) then
					RegTTCdata(15 downto 8) <= SubAddr_q(7 downto 0);
					RegTTCdata( 7 downto 0) <= Dout_q(7 downto 0);

		   	end if;
		   end if;
		end if;
	end process;




  	-- Register TTC Select - Reg28
  	------------------------------

	process (rst, clk)
	begin
		if (rst = '1') then
			RegTTCselect <= "0000";

		elsif rising_edge(clk) then

			if (Reg28 and vmeWR) = '1' then
				RegTTCselect <= db(3 downto 0);

			end if;
      end if;
	end process;



	---------------------------------------------------------------------------
  	-- Command and Control
  	---------------------------------------------------------------------------


	-- Command Reg (High)
	--------------------------------------
	RegTTCCmd(15)  <= '0';
	
	process (rst, clk) --RegTTCCmd(14 downto 8), Reg2E, vmeWR, trx_ttcready_i,
            --SinErrStr_q, DbErrStr_q, DoutStr_q, BrcstStr2_q, BrcstStr1_q, L1Accept_q)
	begin
		if (rst = '1') then
			RegTTCcmd(14 downto 8) <= (others => '0');

		
		elsif rising_edge(clk) then
		
			if (Reg2E and vmeWR) = '1' then
				RegTTCcmd(14 downto 8) <= (others => '0');

			else
	   		RegTTCCmd(14)  <= trx_TTCReady_i;
		   	RegTTCCmd(13)  <= SinErrStr_q  or RegTTCCmd(13);
		   	RegTTCCmd(12)  <= DbErrStr_q   or RegTTCCmd(12);
		   	RegTTCCmd(11)  <= DoutStr_q    or RegTTCCmd(11);
		   	RegTTCCmd(10)  <= BrcstStr2_q  or RegTTCCmd(10);
		   	RegTTCCmd( 9)  <= BrcstStr1_q  or RegTTCCmd( 9);
		   	RegTTCCmd( 8)  <= L1Accept_q   or RegTTCCmd( 8);

	   	end if;
	   end if;
	end process;


	-- Command Reg (Low)
	--------------------
	process (rst, clk)
	begin
		if (rst ='1') then
			RegTTCCmd(7 downto 0) <= "00000000";

		elsif rising_edge(clk) then

			if (BrcstStr2_q = '1') then
				RegTTCCmd(7 downto 6) 	<= Brcst_q(7 downto 6);
		   end if;

		   if (BrcstStr1_q = '1') then
				RegTTCCmd(5 downto 2) 	<= Brcst_q(5 downto 2);
				RegTTCCmd(1) 				<= EvCntRes_q;
				RegTTCCmd(0) 				<= BCntRes_q;
		   end if;

		end if;
	end process;



	---------------------------------------------------------------------------
  	-- I2C Section
  	---------------------------------------------------------------------------

	-- Register I2C - Reg2C
  	-----------------------
	process (rst, clk)
	begin
		if (rst = '1') then
			RegI2C <= "0000000000000000";

		elsif rising_edge(clk) then

			if (Reg2C and vmeWR) = '1' then
				RegI2C <= db(15 downto 0);

			end if;
      end if;
	end process;

	--

	-- I2C is a pull-up bus
	--*** adding logic here for PAR
	trx_SDA_io <= RegI2C(15) when (RegI2C(14) = '1') else 'Z'; -- =1 : I2C is a pull-up bus
	trx_SCL_io <= RegI2C(13) when (RegI2C(12) = '1') else 'Z';



	---------------------------------------------------------------------------
  	-- Read Data
  	---------------------------------------------------------------------------

	read_db : block
	
		signal RegTTCsel_db 	:	std_logic_vector(15 downto 0);
				
	begin
	
		RegTTCsel_db(15 downto 4) <= (others => '0');
		RegTTCsel_db( 3 downto 0) <= RegTTCselect;
  	
  		db <= RegTTCdata 	 when ((vmeRD and Reg26) = '1') else
				RegTTCsel_db when ((vmeRD and Reg28) = '1') else
				RegI2C		 when ((vmeRD and Reg2C) = '1') else
				RegTTCCmd	 when ((vmeRD and Reg2E) = '1') else (others => 'Z');
				
	end block;


	-- Debug Bus
  	----------------------------------------------------
	DebugBus <= "0000000000000000";

end tim;

-------------------------------------------------------------------------------
