-- hds header_start
--
-- VHDL Architecture CALICE_lib.addr_decoder.rtl2
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 09:52:36 11/30/03
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library TIM_fpga2;
use TIM_fpga2.f2_globals.all;


entity addr_decoder is
   port( 
      rst     : in     std_logic;
      clk     : in     std_logic;
      enable  : in     std_logic;
      addr    : in     std_logic_vector (15 downto 0);
      --addr_latch : in     std_logic;
      ram8_en : out    std_logic;
      ren2_o  : out    std_logic_vector (127 downto 0);
      f2_sel  : out    std_logic;
      f1_sel  : out    std_logic;
      ren1_o  : out    std_logic_vector (127 downto 0);
      ram1_en : out    std_logic;
      ab16_o  : out    std_logic_vector (15 downto 0)
   );

-- Declarations

end addr_decoder ;


-- hds interface_end
architecture rtl of addr_decoder is

	signal enable_q		: std_logic;
	
	signal f2_sel_a		: std_logic;
	signal ren2_a 		: std_logic;
	signal ram8_en_a 	: std_logic;
	            
	signal f1_sel_a 		: std_logic;
	signal ren1_a 	: std_logic;
	signal ram1_en_a 	: std_logic;
	
 	signal iaddr_lo 			: integer range 0 to 16#7F#; 	 	            	
 	signal iaddr_hi 			: integer range 0 to 16#FF#; 	 	            	


begin

	iaddr_lo <= CONV_INTEGER( addr(7  downto 1) ); -- register addresses
	iaddr_hi <= CONV_INTEGER( addr(15 downto 8) ); -- address decoding
	
	-- Addr Latch
	--process (addr, addr_latch, rst)
	--begin
		--if (rst = '1') then
			--ab16_o <= (others => '0');
	
		--else
			--if (addr_latch = '0') then
				--ab16_o <= addr after 1 ns;
			--end if;
		--end if;
	--end process;
	
	
	-- Decoder
	process (clk)	
	begin
 		
  		if rising_edge(clk) then

  			if (rst = '1') or (enable = '0') then

 				enable_q	<= '0';

 				ab16_o	<= (others => '0');

				f2_sel	<= '0';
				ren2_o	<= (others => '0');
				ram8_en 	<= '0';

				f1_sel 	<= '0';
				ren1_o	<= (others => '0');
 				ram1_en 	<= '0';
	  		
	    	else
   	  		enable_q <= enable after 1 ns;

 				-- detects rising edge of enable
	   	 	if (enable_q = '0') and (enable = '1') then
	   	 	
	 				ab16_o	<= addr after 1 ns;
			
	 				-- FPGA2
	 				--------------------------------------
	 				f2_sel	<= f2_sel_a after 1 ns;
					ram8_en 	<= ram8_en_a after 1 ns;
	 				
					-- default
					ren2_o 	<= (others => '0') after 1 ns;
	 				
	 				if (ren2_a = '1') then ren2_o(iaddr_lo) <= '1' after 1 ns; end if; 	
										                                    
	 				-- FPGA1
	 				--------------------------------------
		   	 	f1_sel	<= f1_sel_a and (not f2_sel_a) after 1 ns;	
					ram1_en 	<= ram1_en_a;
					
					-- default
					ren1_o 	<= (others => '0');
					
					if (ren1_a = '1') then ren1_o(iaddr_lo) <= '1' after 1 ns; end if; 	

				end if;				
    		end if;
		end if;
  	end process;
  	

	--------------------------------------------------------------------------

	
	-- *** need 1 clock delay in the sel lines - easiest to just clock these:
	--process (clk)
	--begin
		--if (rst = '1') then 
			--f2_sel_a	 <= '0';
			--ren2_a <= '0';
			--ram8_en_a <= '0';
	
			--f1_sel_a	 <= '0';
			--ren1_a <= '0';
			--ram1_en_a <= '0';
		
		--else
	

		async_decode : process (iaddr_hi)
		begin
		 
			-- defaults
			f2_sel_a	 <= '0';
			ren2_a <= '0';
			ram8_en_a <= '0';
		
			f1_sel_a	 <= '0';
			ren1_a <= '0';
			ram1_en_a <= '0';
			
			case iaddr_hi is
	
	      	when 16#00#				 => f2_sel_a <= '1'; ren2_a    <= '1' after 1 ns; -- reg range 0 to 127   (0-0xFE addr)
			  	when 16#01#				 => f1_sel_a <= '1'; ren1_a    <= '1' after 1 ns; -- reg range 128 to 255 (0x100-0x1FE addr)
	
				when 16#80# to 16#FF# => f2_sel_a <= '1'; ram8_en_a <= '1' after 1 ns; -- ram above 0x8000 enable
			 	when 16#10# to 16#1F# => f1_sel_a <= '1'; ram1_en_a <= '1' after 1 ns; -- ram 0x1000 to 0x17FF enable
				
				when others					=> null;
			end case;
		--end if;
	end process;

end rtl;
