-- hds header_start
--
-- VHDL Architecture TIM_lib.reset_gen.rtl
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 11:42:59 10/06/03
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

-- Reset Timer:
-- Waits for other things (like VME interface to settle) before issuing reset


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity reset_gen is
   generic( 
      RST_DEADTIME : integer := 255
   );
   port( 
      clk_in       : in     std_logic;
      reset1_i     : in     std_logic;
      reset2_i     : in     std_logic;
      reset3_i     : in     std_logic;
      dll_locked_i : in     std_logic;
      dll_reset_o  : out    std_logic;
      reset_o      : out    std_logic;
      cfg_rst_o    : out    std_logic;
      init_i       : in     std_logic;
      reset4_i     : in     std_logic
   );

-- Declarations

end reset_gen ;


-- hds interface_end
architecture rtl of reset_gen is

	signal reset_in   		: std_logic;
	signal reset_inq 			: std_logic_vector(2 downto 0);
	signal reset_in_rising  : std_logic;
	signal rst_timer  		: std_logic_vector(7 downto 0);
	signal rst_timer_max		: std_logic_vector(7 downto 0);
	signal rst_timer_en 		: std_logic;

	signal dll_locked_q 		: std_logic;
	signal dll_locked_rising  : std_logic;


	-- State Machine stuff;

	type	states is (sm_START, sm_BEGIN_RESET, sm_SKIP_WAIT_DLL_LOCK, sm_RESET_STRETCH1, sm_RESET_STRETCH2, sm_RESET_STRETCH3, sm_DONE_IDLE);
	signal sm_state 			: states;


begin

	cfg_rst_o <= init_i;

	-- synchronise reset in signal
	------------------------------------------------------------------

	reset_in <= reset1_i or reset2_i or reset3_i or reset4_i;

	sync_edge_det : process (clk_in)
	begin
		if rising_edge(clk_in) then
			if (init_i = '1') then
				-- *** new idea: reset the dll only once - at configure
				dll_reset_o 		<= '1';

				reset_inq 		 	<= (others => '0');
				reset_in_rising 	<= '0';

				dll_locked_q 		<= '0';
				dll_locked_rising <= '0';

			else
				dll_reset_o 		<= '0';

				reset_inq 		 <= (reset_inq(1 downto 0) & reset_in) after 1 ns;
				reset_in_rising <= reset_inq(1) and (not reset_inq(2));

				dll_locked_q 		<= dll_locked_i;
				dll_locked_rising <= dll_locked_i and (not dll_locked_q);



			end if;
		end if;
	end process;


	-------------------------------------------------------------
	reset_gen_machine : process (clk_in)
	begin
		if rising_edge(clk_in) then
			if (init_i = '1') then

				rst_timer_en 	 	<= '0';
				sm_state 			<= sm_START;

	 		else

				-- defaults
				--dll_reset_o  <= '0';
				rst_timer_en <= '0';
				reset_o 		 <= '0';


				-- Reset State Machine
				------------------------
				case sm_state is
		   		when sm_START =>  -- start cycle/wait dead time
						-- default
						sm_state <= sm_START;

						rst_timer_en  <= '1' after 1 ns;

	   				if (rst_timer = rst_timer_max) then
		   				sm_state <= sm_BEGIN_RESET;

		   			end if;


		   		when sm_BEGIN_RESET =>
			  			reset_o		<= '1' after 1 ns;
				   	--dll_reset_o <= '1' after 1 ns;
	   				sm_state 	<= sm_SKIP_WAIT_DLL_LOCK;


		   		when sm_SKIP_WAIT_DLL_LOCK =>
			  			reset_o	<= '1' after 1 ns;

		   			-- default
	   				--sm_state <= sm_WAIT_DLL_LOCK;

		   			--if (dll_locked_rising = '1') then
		   				--sm_state <= sm_DONE_IDLE;
		   			--end if;

		   			sm_state <= sm_RESET_STRETCH1;

		   		when sm_RESET_STRETCH1 =>
			  			reset_o	<= '1' after 1 ns;

	   				sm_state <= sm_RESET_STRETCH2;

		   		when sm_RESET_STRETCH2 =>
			  			reset_o	<= '1' after 1 ns;

	   				sm_state <= sm_RESET_STRETCH3;

		   		when sm_RESET_STRETCH3 =>
			  			reset_o	<= '1' after 1 ns;

	   				sm_state <= sm_DONE_IDLE;

		   		when sm_DONE_IDLE =>  -- rest position
			  			-- default
			  			sm_state <= sm_DONE_IDLE;

			  			if (reset_in_rising = '1') then
			  				sm_state <= sm_START;
			  			end if;

			  		when others =>
						--dll_reset_o  <= '0';
						rst_timer_en <= '0';
     					reset_o 		 <= '0';
	  					sm_state 	 <= sm_DONE_IDLE;

	   		end case;

	   	end if;
		end if;
 	end process;



	-------------------------------------------------------------------------
	rst_timer_max 	<= conv_std_logic_vector(RST_DEADTIME, 8);

	reset_timer : process (clk_in)
	begin
		if rising_edge(clk_in) then
			if (init_i = '1') or (rst_timer_en = '0')then
				rst_timer <= (others => '0');

			else
				rst_timer <= rst_timer + '1' after 1 ns;

		   end if;
		end if;
 	end process;


end rtl;
